@@ -3,7 +3,7 @@
  *  file console.c
  *  Copyright (C) 1998--2003  Guido Masarotto and Brian Ripley
  *  Copyright (C) 2004-8      The R Foundation
- *  Copyright (C) 2004-13     The R Core Team
+ *  Copyright (C) 2004-2013   The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -278,18 +278,17 @@ static size_t enctowcs(wchar_t *wc, char *s, int n)
 
 static void xbufadds(xbuf p, const char *s, int user)
 {
-    int n = strlen(s) + 1; /* UCS-2 must be shorter */
+    int n = strlen(s) + 1; /* UCS-2 must be no more chars */
     if (n < 1000) {
 	wchar_t tmp[n];
 	enctowcs(tmp, (char *) s, n);
 	xbufaddxs(p, tmp, user);
     } else {
 	/* very long line */
-	void *vmax = vmaxget();
-	wchar_t *tmp = (wchar_t*) R_alloc(n, sizeof(wchar_t));
+	wchar_t *tmp = (wchar_t*) malloc(n * sizeof(wchar_t));
 	enctowcs(tmp, (char *) s, n);
 	xbufaddxs(p, tmp, user);
-	vmaxset(vmax);
+	free(tmp);
     }
 }
 
