@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1999-2010   The R Core Team.
+ *  Copyright (C) 1999-2013   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -38,30 +38,49 @@ extern "C" {
 #include <R_ext/Complex.h>
 #include <R_ext/Error.h>
 #include <R_ext/Memory.h>
-#include <R_ext/PrtUtil.h>
 #include <R_ext/Utils.h>
+#include <R_ext/Print.h>
 
 #include <R_ext/libextern.h>
 
 typedef unsigned char Rbyte;
 
-/* type for length of vectors etc */
-typedef int R_len_t; /* will be long later, LONG64 or ssize_t on Win64 */
+/* type for length of (standard, not long) vectors etc */
+typedef int R_len_t;
 #define R_LEN_T_MAX INT_MAX
 
+/* both config.h and Rconfig.h set SIZEOF_SIZE_T, but Rconfig.h is
+   skipped if config.h has already been included. */
+#ifndef R_CONFIG_H
+# include <Rconfig.h>
+#endif
+
+#if ( SIZEOF_SIZE_T > 4 )
+# define LONG_VECTOR_SUPPORT
+#endif
+
+#ifdef LONG_VECTOR_SUPPORT
+    typedef ptrdiff_t R_xlen_t;
+    typedef struct { R_xlen_t lv_length, lv_truelength; } R_long_vec_hdr_t;
+# define R_XLEN_T_MAX 4503599627370496
+# define R_SHORT_LEN_MAX 2147483647
+# define R_LONG_VEC_TOKEN -1
+#else
+    typedef int R_xlen_t;
+# define R_XLEN_T_MAX R_LEN_T_MAX
+#endif
+
+
 /* Fundamental Data Types:  These are largely Lisp
  * influenced structures, with the exception of LGLSXP,
  * INTSXP, REALSXP, CPLXSXP and STRSXP which are the
  * element types for S-like data objects.
-
- * Note that the gap of 11 and 12 below is because of
- * the withdrawal of native "factor" and "ordered" types.
  *
  *			--> TypeTable[] in ../main/util.c for  typeof()
  */
 
 /*  These exact numeric values are seldom used, but they are, e.g., in
- *  ../main/subassign.c
+ *  ../main/subassign.c, and they are serialized.
 */
 #ifndef enum_SEXPTYPE
 /* NOT YET using enum:
@@ -85,6 +104,7 @@ typedef unsigned int SEXPTYPE;
 #define BUILTINSXP   8	  /* builtin non-special forms */
 #define CHARSXP	     9	  /* "scalar" string type (internal only)*/
 #define LGLSXP	    10	  /* logical vectors */
+/* 11 and 12 were factors and ordered factors in the 1990s */
 #define INTSXP	    13	  /* integer vectors */
 #define REALSXP	    14	  /* real variables */
 #define CPLXSXP	    15	  /* complex variables */
@@ -255,19 +275,55 @@ typedef union { VECTOR_SEXPREC s; double align; } SEXPREC_ALIGN;
 #define SET_TYPEOF(x,v)	(((x)->sxpinfo.type)=(v))
 #define SET_NAMED(x,v)	(((x)->sxpinfo.named)=(v))
 #define SET_RTRACE(x,v)	(((x)->sxpinfo.trace)=(v))
-#define SETLEVELS(x,v)	(((x)->sxpinfo.gp)=(v))
+#define SETLEVELS(x,v)	(((x)->sxpinfo.gp)=((unsigned short)v))
 
 /* S4 object bit, set by R_do_new_object for all new() calls */
-#define S4_OBJECT_MASK (1<<4)
+#define S4_OBJECT_MASK ((unsigned short)(1<<4))
 #define IS_S4_OBJECT(x) ((x)->sxpinfo.gp & S4_OBJECT_MASK)
 #define SET_S4_OBJECT(x) (((x)->sxpinfo.gp) |= S4_OBJECT_MASK)
 #define UNSET_S4_OBJECT(x) (((x)->sxpinfo.gp) &= ~S4_OBJECT_MASK)
 
 /* Vector Access Macros */
-#define LENGTH(x)	(((VECSEXP) (x))->vecsxp.length)
-#define TRUELENGTH(x)	(((VECSEXP) (x))->vecsxp.truelength)
-#define SETLENGTH(x,v)		((((VECSEXP) (x))->vecsxp.length)=(v))
-#define SET_TRUELENGTH(x,v)	((((VECSEXP) (x))->vecsxp.truelength)=(v))
+#ifdef LONG_VECTOR_SUPPORT
+    R_len_t R_BadLongVector(SEXP, const char *, int);
+# define IS_LONG_VEC(x) (SHORT_VEC_LENGTH(x) == R_LONG_VEC_TOKEN)
+# define SHORT_VEC_LENGTH(x) (((VECSEXP) (x))->vecsxp.length)
+# define SHORT_VEC_TRUELENGTH(x) (((VECSEXP) (x))->vecsxp.truelength)
+# define LONG_VEC_LENGTH(x) ((R_long_vec_hdr_t *) (x))[-1].lv_length
+# define LONG_VEC_TRUELENGTH(x) ((R_long_vec_hdr_t *) (x))[-1].lv_truelength
+# define XLENGTH(x) (IS_LONG_VEC(x) ? LONG_VEC_LENGTH(x) : SHORT_VEC_LENGTH(x))
+# define XTRUELENGTH(x)	(IS_LONG_VEC(x) ? LONG_VEC_TRUELENGTH(x) : SHORT_VEC_TRUELENGTH(x))
+# define LENGTH(x) (IS_LONG_VEC(x) ? R_BadLongVector(x, __FILE__, __LINE__) : SHORT_VEC_LENGTH(x))
+# define TRUELENGTH(x) (IS_LONG_VEC(x) ? R_BadLongVector(x, __FILE__, __LINE__) : SHORT_VEC_TRUELENGTH(x))
+# define SET_SHORT_VEC_LENGTH(x,v) (SHORT_VEC_LENGTH(x) = (v))
+# define SET_SHORT_VEC_TRUELENGTH(x,v) (SHORT_VEC_TRUELENGTH(x) = (v))
+# define SET_LONG_VEC_LENGTH(x,v) (LONG_VEC_LENGTH(x) = (v))
+# define SET_LONG_VEC_TRUELENGTH(x,v) (LONG_VEC_TRUELENGTH(x) = (v))
+# define SETLENGTH(x,v) do { \
+      SEXP sl__x__ = (x); \
+      R_xlen_t sl__v__ = (v); \
+      if (IS_LONG_VEC(sl__x__)) \
+	  SET_LONG_VEC_LENGTH(sl__x__,  sl__v__); \
+      else SET_SHORT_VEC_LENGTH(sl__x__, (R_len_t) sl__v__); \
+  } while (0)
+# define SET_TRUELENGTH(x,v) do { \
+      SEXP sl__x__ = (x); \
+      R_xlen_t sl__v__ = (v); \
+      if (IS_LONG_VEC(sl__x__)) \
+	  SET_LONG_VEC_TRUELENGTH(sl__x__, sl__v__); \
+      else SET_SHORT_VEC_TRUELENGTH(sl__x__, (R_len_t) sl__v__); \
+  } while (0)
+#else
+# define LENGTH(x)	(((VECSEXP) (x))->vecsxp.length)
+# define TRUELENGTH(x)	(((VECSEXP) (x))->vecsxp.truelength)
+# define XLENGTH(x) LENGTH(x)
+# define XTRUELENGTH(x) TRUELENGTH(x)
+# define SETLENGTH(x,v)		((((VECSEXP) (x))->vecsxp.length)=(v))
+# define SET_TRUELENGTH(x,v)	((((VECSEXP) (x))->vecsxp.truelength)=(v))
+# define SET_SHORT_VEC_LENGTH SETLENGTH
+# define SET_SHORT_VEC_TRUELENGTH SET_TRUELENGTH
+# define IS_LONG_VEC(x) 0
+#endif
 
 /* Under the generational allocator the data for vector nodes comes
    immediately after the node structure, so the data address is a
@@ -352,6 +408,13 @@ Rboolean (Rf_isObject)(SEXP s);
 
 #endif /* USE_RINTERNALS */
 
+/* Macros for common NAMED and SET_NAMED idioms. */
+#define NAMEDMAX 2
+#define MAYBE_SHARED(x) (NAMED(x) > 1)
+#define NO_REFERENCES(x) (NAMED(x) == 0)
+#define MARK_NOT_MUTABLE(x) SET_NAMED(x, NAMEDMAX)
+#define MAYBE_REFERENCED(x) (! NO_REFERENCES(x))
+
 /* Accessor functions.  Many are declared using () to avoid the macro
    definitions in the USE_RINTERNALS section.
    The function STRING_ELT is used as an argument to arrayAssign even
@@ -380,6 +443,9 @@ int  (LENGTH)(SEXP x);
 int  (TRUELENGTH)(SEXP x);
 void (SETLENGTH)(SEXP x, int v);
 void (SET_TRUELENGTH)(SEXP x, int v);
+R_xlen_t  (XLENGTH)(SEXP x);
+R_xlen_t  (XTRUELENGTH)(SEXP x);
+int  (IS_LONG_VEC)(SEXP x);
 int  (LEVELS)(SEXP x);
 int  (SETLEVELS)(SEXP x, int v);
 
@@ -388,10 +454,10 @@ int  *(INTEGER)(SEXP x);
 Rbyte *(RAW)(SEXP x);
 double *(REAL)(SEXP x);
 Rcomplex *(COMPLEX)(SEXP x);
-SEXP (STRING_ELT)(SEXP x, int i);
-SEXP (VECTOR_ELT)(SEXP x, int i);
-void SET_STRING_ELT(SEXP x, int i, SEXP v);
-SEXP SET_VECTOR_ELT(SEXP x, int i, SEXP v);
+SEXP (STRING_ELT)(SEXP x, R_xlen_t i);
+SEXP (VECTOR_ELT)(SEXP x, R_xlen_t i);
+void SET_STRING_ELT(SEXP x, R_xlen_t i, SEXP v);
+SEXP SET_VECTOR_ELT(SEXP x, R_xlen_t i, SEXP v);
 SEXP *(STRING_PTR)(SEXP x);
 SEXP *(VECTOR_PTR)(SEXP x);
 
@@ -581,7 +647,7 @@ SEXP Rf_allocMatrix(SEXPTYPE, int, int);
 SEXP Rf_allocList(int);
 SEXP Rf_allocS4Object(void);
 SEXP Rf_allocSExp(SEXPTYPE);
-SEXP Rf_allocVector(SEXPTYPE, R_len_t);
+SEXP Rf_allocVector(SEXPTYPE, R_xlen_t);
 int  Rf_any_duplicated(SEXP x, Rboolean from_last);
 int  Rf_any_duplicated3(SEXP x, SEXP incomp, Rboolean from_last);
 SEXP Rf_applyClosure(SEXP, SEXP, SEXP, SEXP, SEXP);
@@ -600,6 +666,7 @@ SEXP Rf_dimgets(SEXP, SEXP);
 SEXP Rf_dimnamesgets(SEXP, SEXP);
 SEXP Rf_DropDims(SEXP);
 SEXP Rf_duplicate(SEXP);
+/* the next really should not be here and is also in Defn.h */
 SEXP Rf_duplicated(SEXP, Rboolean);
 SEXP Rf_eval(SEXP, SEXP);
 SEXP Rf_findFun(SEXP, SEXP);
@@ -622,6 +689,7 @@ Rboolean Rf_isOrdered(SEXP);
 Rboolean Rf_isUnordered(SEXP);
 Rboolean Rf_isUnsorted(SEXP, Rboolean);
 SEXP Rf_lengthgets(SEXP, R_len_t);
+SEXP Rf_xlengthgets(SEXP, R_xlen_t);
 SEXP R_lsInternal(SEXP, Rboolean);
 SEXP Rf_match(SEXP, SEXP, int);
 SEXP Rf_matchE(SEXP, SEXP, int, SEXP);
@@ -701,6 +769,7 @@ void R_RegisterFinalizer(SEXP s, SEXP fun);
 void R_RegisterCFinalizer(SEXP s, R_CFinalizer_t fun);
 void R_RegisterFinalizerEx(SEXP s, SEXP fun, Rboolean onexit);
 void R_RegisterCFinalizerEx(SEXP s, R_CFinalizer_t fun, Rboolean onexit);
+void R_RunPendingFinalizers(void);
 
 /* Weak reference interface */
 SEXP R_MakeWeakRef(SEXP key, SEXP val, SEXP fin, Rboolean onexit);
@@ -740,7 +809,11 @@ Rboolean R_HasFancyBindings(SEXP rho);
 
 /* ../main/errors.c : */
 /* needed for R_load/savehistory handling in front ends */
+#if defined(__GNUC__) && __GNUC__ >= 3
+void Rf_errorcall(SEXP, const char *, ...) __attribute__((noreturn));
+#else
 void Rf_errorcall(SEXP, const char *, ...);
+#endif
 void Rf_warningcall(SEXP, const char *, ...);
 void Rf_warningcall_immediate(SEXP, const char *, ...);
 
@@ -802,7 +875,7 @@ void R_InitFileOutPStream(R_outpstream_t stream, FILE *fp,
 			  SEXP (*phook)(SEXP, SEXP), SEXP pdata);
 
 #ifdef NEED_CONNECTION_PSTREAMS
-/* The connection interface is not yet available to packages.  To
+/* The connection interface is not available to packages.  To
    allow limited use of connection pointers this defines the opaque
    pointer type. */
 #ifndef HAVE_RCONNECTION_TYPEDEF
@@ -857,6 +930,10 @@ int R_system(const char *);
 */
 Rboolean R_compute_identical(SEXP, SEXP, int);
 
+/* C version of R's  indx <- order(..., na.last, decreasing) :
+   e.g.  arglist = Rf_lang2(x,y)  or  Rf_lang3(x,y,z) */
+void R_orderVector(int *indx, int n, SEXP arglist, Rboolean nalast, Rboolean decreasing);
+
 #ifndef R_NO_REMAP
 #define acopy_string		Rf_acopy_string
 #define alloc3DArray            Rf_alloc3DArray
@@ -1009,6 +1086,8 @@ Rboolean R_compute_identical(SEXP, SEXP, int);
 #define VectorToPairList	Rf_VectorToPairList
 #define warningcall		Rf_warningcall
 #define warningcall_immediate	Rf_warningcall_immediate
+#define xlength(x)		Rf_xlength(x)
+#define xlengthgets		Rf_xlengthgets
 
 #endif
 
@@ -1070,6 +1149,7 @@ SEXP	 Rf_ScalarLogical(int);
 SEXP	 Rf_ScalarRaw(Rbyte);
 SEXP	 Rf_ScalarReal(double);
 SEXP	 Rf_ScalarString(SEXP);
+R_xlen_t  Rf_xlength(SEXP);
 #endif
 
 #ifdef USE_RINTERNALS
