@@ -1,7 +1,7 @@
 #  File src/library/base/R/dates.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -71,8 +71,10 @@ as.Date.default <- function(x, ...)
     if(inherits(x, "Date")) return(x)
     if(is.logical(x) && all(is.na(x)))
         return(structure(as.numeric(x), class = "Date"))
-    stop(gettextf("do not know how to convert '%s' to class \"Date\"",
-                  deparse(substitute(x))))
+    stop(gettextf("do not know how to convert '%s' to class %s",
+                  deparse(substitute(x)),
+                  dQuote("Date")),
+         domain = NA)
 }
 
 ## convert from package date
@@ -117,7 +119,7 @@ print.Date <- function(x, max = NULL, ...)
     invisible(x)
 }
 
-summary.Date <- function(object, digits = 12, ...)
+summary.Date <- function(object, digits = 12L, ...)
 {
     x <- summary.default(unclass(object), digits = digits, ...)
     if(m <- match("NA's", names(x), 0)) {
@@ -140,7 +142,7 @@ summary.Date <- function(object, digits = 12, ...)
     if (nargs() == 1) return(e1)
     # only valid if one of e1 and e2 is a scalar.
     if(inherits(e1, "Date") && inherits(e2, "Date"))
-        stop("binary + is not defined for Date objects")
+        stop("binary + is not defined for \"Date\" objects")
     if (inherits(e1, "difftime")) e1 <- coerceTimeUnit(e1)
     if (inherits(e2, "difftime")) e2 <- coerceTimeUnit(e2)
     structure(unclass(e1) + unclass(e2), class = "Date")
@@ -153,23 +155,26 @@ summary.Date <- function(object, digits = 12, ...)
                                secs = x/86400, mins = x/1440, hours = x/24,
                                days = x, weeks = 7*x)))
     if(!inherits(e1, "Date"))
-        stop("Can only subtract from Date objects")
-    if (nargs() == 1) stop("unary - is not defined for Date objects")
+        stop("can only subtract from \"Date\" objects")
+    if (nargs() == 1) stop("unary - is not defined for \"Date\" objects")
     if(inherits(e2, "Date")) return(difftime(e1, e2, units="days"))
     if (inherits(e2, "difftime")) e2 <- coerceTimeUnit(e2)
     if(!is.null(attr(e2, "class")))
-        stop("can only subtract numbers from Date objects")
+        stop("can only subtract numbers from \"Date\" objects")
     structure(unclass(as.Date(e1)) - e2, class = "Date")
 }
 
 Ops.Date <- function(e1, e2)
 {
     if (nargs() == 1)
-        stop("unary ", .Generic, " not defined for Date objects")
+        stop(gettextf("unary %s not defined for \"Date\" objects", .Generic),
+             domain = NA)
     boolean <- switch(.Generic, "<" =, ">" =, "==" =,
                       "!=" =, "<=" =, ">=" = TRUE,
                       FALSE)
-    if (!boolean) stop(.Generic, " not defined for Date objects")
+    if (!boolean)
+        stop(gettextf("%s not defined for \"Date\" objects", .Generic),
+             domain = NA)
     ## allow character args to be coerced to dates
     if (is.character(e1)) e1 <- as.Date(e1)
     if (is.character(e2)) e2 <- as.Date(e2)
@@ -177,13 +182,15 @@ Ops.Date <- function(e1, e2)
 }
 
 Math.Date <- function (x, ...)
-    stop(.Generic, " not defined for Date objects")
+    stop(gettextf("%s not defined for \"Date\" objects", .Generic),
+         domain = NA)
 
 Summary.Date <- function (..., na.rm)
 {
     ok <- switch(.Generic, max = , min = , range = TRUE, FALSE)
-    if (!ok) stop(.Generic, " not defined for Date objects")
-    val <- NextMethod(.Generic)
+    if (!ok) stop(gettextf("%s not defined for \"Date\" objects", .Generic),
+                  domain = NA)
+   val <- NextMethod(.Generic)
     class(val) <- oldClass(list(...)[[1L]])
     val
 }
@@ -224,19 +231,19 @@ as.data.frame.Date <- as.data.frame.vector
 as.list.Date <- function(x, ...)
     lapply(seq_along(x), function(i) x[i])
 
-c.Date <- function(..., recursive=FALSE)
-    structure(c(unlist(lapply(list(...), unclass))), class="Date")
+c.Date <- function(..., recursive = FALSE)
+    structure(c(unlist(lapply(list(...), unclass))), class = "Date")
 
 mean.Date <- function (x, ...)
     structure(mean(unclass(x), ...), class = "Date")
 
-seq.Date <- function(from, to, by, length.out=NULL, along.with=NULL, ...)
+seq.Date <- function(from, to, by, length.out = NULL, along.with = NULL, ...)
 {
     if (missing(from)) stop("'from' must be specified")
-    if (!inherits(from, "Date")) stop("'from' must be a Date object")
+    if (!inherits(from, "Date")) stop("'from' must be a \"Date\" object")
         if(length(as.Date(from)) != 1L) stop("'from' must be of length 1")
     if (!missing(to)) {
-        if (!inherits(to, "Date")) stop("'to' must be a Date object")
+        if (!inherits(to, "Date")) stop("'to' must be a \"Date\" object")
         if (length(as.Date(to)) != 1L) stop("'to' must be of length 1")
     }
     if (!missing(along.with)) {
@@ -246,7 +253,7 @@ seq.Date <- function(from, to, by, length.out=NULL, along.with=NULL, ...)
         length.out <- ceiling(length.out)
     }
     status <- c(!missing(to), !missing(by), !is.null(length.out))
-    if(sum(status) != 2)
+    if(sum(status) != 2L)
         stop("exactly two of 'to', 'by' and 'length.out' / 'along.with' must be specified")
     if (missing(by)) {
         from <- unclass(as.Date(from))
@@ -285,10 +292,10 @@ seq.Date <- function(from, to, by, length.out=NULL, along.with=NULL, ...)
             res <- seq.int(0, to0 - from, by) + from
         }
         res <- structure(res, class="Date")
-    } else {  # months or years or DSTdays
+    } else {  # months or years
         r1 <- as.POSIXlt(from)
-        if(valid == 4L) {
-            if(missing(to)) { # years
+        if(valid == 4L) { # years
+            if(missing(to)) {
                 yr <- seq.int(r1$year, by = by, length.out = length.out)
             } else {
                 to0 <- as.POSIXlt(to)
@@ -328,7 +335,7 @@ cut.Date <-
     } else if(is.numeric(breaks) && length(breaks) == 1L) {
 	## specified number of breaks
     } else if(is.character(breaks) && length(breaks) == 1L) {
-	by2 <- strsplit(breaks, " ", fixed=TRUE)[[1L]]
+	by2 <- strsplit(breaks, " ", fixed = TRUE)[[1L]]
 	if(length(by2) > 2L || length(by2) < 1L)
 	    stop("invalid specification of 'breaks'")
 	valid <-
@@ -439,7 +446,7 @@ diff.Date <- function (x, lag = 1L, differences = 1L, ...)
 {
     ismat <- is.matrix(x)
     xlen <- if (ismat) dim(x)[1L] else length(x)
-    if (length(lag) > 1L || length(differences) > 1L || lag < 1L || differences < 1L)
+    if (length(lag) != 1L || length(differences) > 1L || lag < 1L || differences < 1L)
         stop("'lag' and 'differences' must be integers >= 1")
     if (lag * differences >= xlen)
         return(structure(numeric(), class="difftime", units="days"))
