@@ -1,6 +1,8 @@
 #  File src/library/base/R/findInt.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2013 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -17,30 +19,14 @@
 ### This is a `variant' of  approx( method = "constant" ) :
 findInterval <- function(x, vec, rightmost.closed = FALSE, all.inside = FALSE)
 {
-    ## Purpose: gives back the indices of  x in vec;  vec[] sorted
-    ## -------------------------------------------------------------------------
-    ## Author: Martin Maechler, Date:  4 Jan 2002, 10:16
+    ## Purpose: returns the indices of  x in vec;  vec[] sorted
+    ## ---------------------------------------------------------
+    ## Author: Martin Maechler, Date:  4 Jan 2002, 10:16 (of very different .C version)
 
     if(any(is.na(vec)))
 	stop("'vec' contains NAs")
     if(is.unsorted(vec))
 	stop("'vec' must be sorted non-decreasingly")
-    ## deal with NA's in x:
-    if(has.na <- any(ix <- is.na(x))) x <- x[!ix]
-    nx <- length(x)
-    index <- integer(nx)
-    ## lengths are always integer
-    .C("find_interv_vec",
-       xt = as.double(vec), n = length(vec),
-       x  = as.double(x),  nx = nx,
-       as.logical(rightmost.closed),
-       as.logical(all.inside),
-       index, DUP = FALSE, NAOK = TRUE, # NAOK: 'Inf' only
-       PACKAGE = "base")
-    if(has.na) {
-	ii <- as.integer(ix)
-	ii[ix] <- NA
-	ii[!ix] <- index
-	ii
-    } else index
+    .Internal(findInterval(as.double(vec), as.double(x),
+                           rightmost.closed, all.inside))
 }
