@@ -1,7 +1,7 @@
 #  File src/library/base/R/library.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -213,9 +213,9 @@ function(package, help, pos = 2, lib.loc = NULL, character.only = FALSE,
                                               domain = NA)
                     }
 
-                    objs <- strwrap(paste(same, collapse=", "), indent=4,
-                                    exdent=4)
-                    msg <- sprintf(ngettext(length(objs),
+                    objs <- strwrap(paste(same, collapse=", "),
+                                    indent = 4L, exdent = 4L)
+                    msg <- sprintf(ngettext(length(same),
                                             "The following object is masked %s %s:\n\n%s\n",
                                             "The following objects are masked %s %s:\n\n%s\n"),
                                    if (i < lib.pos) "_by_" else "from",
