@@ -1,6 +1,8 @@
 #  File src/library/base/R/license.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -16,18 +18,20 @@
 
 licence <- license <- function() {
     cat("\nThis software is distributed under the terms of the GNU General\n")
-    cat("Public License Version 2, June 1991.  The terms of this license\n")
-    cat("are in a file called COPYING which you should have received with\n")
+    cat("Public License, either Version 2, June 1991 or Version 3, June 2007.\n")
+    cat("The terms of version 2 of the license are in a file called COPYING\nwhich you should have received with\n")
     cat("this software and which can be displayed by RShowDoc(\"COPYING\").\n")
+    cat("Version 3 of the license can be displayed by RShowDoc(\"GPL-3\").\n")
     cat("\n")
-    cat("If you have not received a copy of this file, you can obtain one\n")
+    cat("Copies of both versions 2 and 3 of the license can be found\n")
     cat("at http://www.R-project.org/Licenses/.\n")
     cat("\n")
     cat("A small number of files (the API header files listed in\n")
     cat("R_DOC_DIR/COPYRIGHTS) are distributed under the\n")
-    cat("Lesser GNU General Public LIcense version 2.1.\n")
-    cat("This can be displayed by RShowDoc(\"COPYING.LIB\"),\n")
+    cat("LESSER GNU GENERAL PUBLIC LICENSE, version 2.1 or later.\n")
+    cat("This can be displayed by RShowDoc(\"LGPL-2.1\"),\n")
     cat("or obtained at the URI given.\n")
+    cat("Version 3 of the license can be displayed by RShowDoc(\"LGPL-3\").\n")
     cat("\n")
     cat("'Share and Enjoy.'\n\n")
 }
