@@ -1,7 +1,7 @@
 #  File src/library/base/R/windows/system.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -52,10 +52,7 @@ system <- function(command, intern = FALSE,
         if(stdout == "") stdout <- TRUE
         if(!ignore.stderr && .Platform$GUI == "Rgui") stderr <- TRUE
     } else {
-        if  (wait)
-            flag <- ifelse(show.output.on.console, 2L, 1L)
-        else
-            flag <- 0L
+        flag <- if (wait) ifelse(show.output.on.console, 2L, 1L) else 0L
     }
     if (invisible) flag <- 20L + flag
     else if (minimized) flag <- 10L + flag
@@ -131,8 +128,4 @@ Sys.timezone <- function()
     if(length(zz) == 3L) zz[2L + z$isdst] else zz[1L]
 }
 
-Sys.which <- function(names)
-{
-    if (any(is.na(names))) stop("missing values are not allowed")
-    .Internal(Sys.which(as.character(names)))
-}
+Sys.which <- function(names) .Internal(Sys.which(as.character(names)))
