@@ -1,7 +1,7 @@
 #  File src/library/grDevices/R/windows/dev2bitmap.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -21,12 +21,13 @@ dev2bitmap <- function(file, type="png16m", height = 7, width = 7, res = 72,
                        method = c("postscript", "pdf"), taa = NA, gaa = NA)
 {
     if(missing(file)) stop("'file' is missing with no default")
-    if(!is.character(file) || length(file) != 1 || !nzchar(file))
+    if(!is.character(file) || length(file) != 1L || !nzchar(file))
         stop("'file' must be a non-empty character string")
     method <- match.arg(method)
     units <- match.arg(units, c("in", "px", "cm", "mm"))
     height <- switch(units, "in"=1, "cm"=1/2.54, "mm"=1/25.4, "px"=1/res) * height
     width <- switch(units, "in"=1, "cm"=1/2.54, "mm"=1/25.4, "px"=1/res) * width
+    ## consider using tools::find_gs_cmd() here
     gsexe <- Sys.getenv("R_GSCMD")
     if(!nzchar(gsexe)) gsexe <- Sys.getenv("GSC")
     if(is.null(gsexe) || !nzchar(gsexe)) {
@@ -42,7 +43,7 @@ dev2bitmap <- function(file, type="png16m", height = 7, width = 7, res = 72,
         warning("unrecognized format of gs -help")
     else {
         gsdevs <- gshelp[(st+1):(en-1)]
-        devs <- c(strsplit(gsdevs, " "), recursive=TRUE)
+        devs <- c(strsplit(gsdevs, " "), recursive = TRUE)
         if(match(type, devs, 0) == 0)
             stop(gettextf("device '%s' is not available\n", type),
                  gettextf("Available devices are %s",
@@ -84,7 +85,7 @@ bitmap <- function(file, type = "png16m", height = 7, width = 7, res = 72,
                    units = "in", pointsize, taa = NA, gaa = NA, ...)
 {
     if(missing(file)) stop("'file' is missing with no default")
-    if(!is.character(file) || length(file) != 1 || !nzchar(file))
+    if(!is.character(file) || length(file) != 1L || !nzchar(file))
         stop("'file' must be a non-empty character string")
     units <- match.arg(units, c("in", "px", "cm", "mm"))
     height <- switch(units, "in"=1, "cm"=1/2.54, "mm"=1/25.4, "px"=1/res) * height
