@@ -1,6 +1,8 @@
 #  File src/library/grDevices/R/zzz.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2013 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -16,32 +18,34 @@
 
 .noGenerics <- TRUE
 
+if (.Platform$OS.type == "windows")
+    utils::globalVariables(c("C_cairoProps", "C_makeQuartzDefault"), add = TRUE)
+
 .onLoad <- function(libname, pkgname)
 {
     op <- options()
     extras <- if(.Platform$OS.type == "windows")
         list(windowsTimeouts = c(100L,500L)) else
     list(bitmapType = if(capabilities("aqua")) "quartz"
-    else if(.Call("cairoProps", 2L, PACKAGE="grDevices")) "cairo" else "Xlib")
-    defdev <- as.vector(Sys.getenv("R_DEFAULT_DEVICE"))
+    else if(.Call(C_cairoProps, 2L)) "cairo" else "Xlib")
+    defdev <- Sys.getenv("R_DEFAULT_DEVICE")
     ## Use devices rather than names to make it harder to get masked.
     if(!nzchar(defdev)) defdev <- pdf
     device <- if(interactive()) {
-        intdev <- as.vector(Sys.getenv("R_INTERACTIVE_DEVICE"))
+        intdev <- Sys.getenv("R_INTERACTIVE_DEVICE")
         if(nzchar(intdev)) intdev
         else {
             dsp <- Sys.getenv("DISPLAY")
             if(.Platform$OS.type == "windows") windows
             else if (.Platform$GUI == "AQUA" ||
                      ((!nzchar(dsp) || grepl("^/tmp/launch-", dsp))
-                      && .Call("makeQuartzDefault"))) quartz
+                      && .Call(C_makeQuartzDefault))) quartz
             else if (nzchar(dsp) && .Platform$GUI %in% c("X11", "Tk")) X11
 	    else defdev
         }
     } else defdev
 
-    if (.Platform$OS.type != "windows"
-        && !.Call("cairoProps", 2L, PACKAGE="grDevices"))
+    if (.Platform$OS.type != "windows" && !.Call(C_cairoProps, 2L))
         X11.options(type = "Xlib")
     op.grDevices <- c(list(locatorBell = TRUE, device.ask.default = FALSE),
                   extras, device = device)
