@@ -1,6 +1,8 @@
 #  File src/library/graphics/R/axis.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2013 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -21,28 +23,27 @@ axis <- function(side, at = NULL, labels = TRUE, tick = TRUE, line = NA,
                  hadj = NA, padj = NA, ...)
 {
     ## we need to do this as the C code processes 'col' before '...'
-    if(is.null(col) && length(list(...)) && !is.null(fg <- list(...)$fg)) {
+    if(is.null(col) && !missing(...) && !is.null(fg <- list(...)$fg))
         ## help(par) 'fg' says this should work
         col <- fg
-    }
     ## watch out: some people do things like
     ## axis(1, at = my.at <- 10^(1L:5), labels = formatC(my.at, format="fg"))
     ## which depends on the order of evaluation of the args.
-    .Internal(axis(side, at, as.graphicsAnnot(labels),
-                   tick, line, pos, outer, font, lty, lwd, lwd.ticks,
-                   col, col.ticks, hadj, padj, ...))
+    invisible(.External.graphics(C_axis, side, at, as.graphicsAnnot(labels),
+          tick, line, pos, outer, font, lty, lwd, lwd.ticks,
+          col, col.ticks, hadj, padj, ...))
 }
 
 
-Axis <- function(x=NULL, at=NULL, ..., side, labels=NULL)
+Axis <- function(x = NULL, at = NULL, ..., side, labels = NULL)
 {
     if (!is.null(x)) UseMethod("Axis", x)
     else if (!is.null(at)) UseMethod("Axis", at)
-    else axis(side=side, at=at, labels=labels, ...)
+    else axis(side = side, at = at, labels = labels, ...)
 }
 
-Axis.default <- function(x=NULL, at=NULL, ..., side, labels=NULL)
-    axis(side=side, at=at, labels=labels, ...)
+Axis.default <- function(x = NULL, at = NULL, ..., side, labels = NULL)
+    axis(side = side, at = at, labels = labels, ...)
 
 Axis.table <- function(x, at, ..., side = 1, labels)
 {
@@ -54,10 +55,7 @@ Axis.table <- function(x, at, ..., side = 1, labels)
          x0 <- if (is.num) xx else seq.int(x)
          if(missing(at)) at <- x0
          if(missing(labels)) labels <- nx
-         xaxt <- if (length(as <- list(...))) {
-             if (!is.null(as$axes) && !as$axes) "n" else as$xaxt
-         }## else NULL
-         axis(side, at = at, labels = labels, xaxt = xaxt)
+         axis(side, at = at, labels = labels, ...)
      }
      else stop("only for 1-D table")
 }
@@ -74,7 +72,7 @@ axTicks <- function(side, axp = NULL, usr = NULL, log = NULL, nintLog = NULL)
     if(!(side <- as.integer(side)) %in% 1L:4L)
         stop("'side' must be in {1:4}")
     is.x <- side %% 2 == 1
-    XY <- function(ch) paste(if(is.x) "x" else "y", ch, sep="")
+    XY <- function(ch) paste0(if(is.x) "x" else "y", ch)
     if(is.null(axp)) axp <- par(XY("axp"))
     else if(!is.numeric(axp) || length(axp) != 3) stop("invalid 'axp'")
     if(is.null(log)) log <- par(XY("log"))
