@@ -1,7 +1,7 @@
 #  File src/library/graphics/R/dotchart.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@ function(x, labels = NULL, groups = NULL, gdata = NULL, cex = par("cex"),
 	    labels <- rownames(x)
 	if (is.null(labels))
 	    labels <- as.character(1L:nrow(x))
-	labels <- rep(labels, length.out = n)
+	labels <- rep_len(labels, n)
 	if (is.null(groups))
 	    groups <- col(x, as.factor = TRUE)
 	glabels <- levels(groups)
@@ -77,8 +77,8 @@ function(x, labels = NULL, groups = NULL, gdata = NULL, cex = par("cex"),
 	o <- sort.list(as.numeric(groups), decreasing = TRUE)
 	x <- x[o]
 	groups <- groups[o]
-	color <- rep(color, length.out=length(groups))[o]
-	lcolor <- rep(lcolor, length.out=length(groups))[o]
+	color <- rep_len(color, length(groups))[o]
+	lcolor <- rep_len(lcolor, length(groups))[o]
 	offset <- cumsum(c(0, diff(as.numeric(groups)) != 0))
 	y <- 1L:n + 2 * offset
 	ylim <- range(0, y + 2)
