@@ -1,6 +1,8 @@
 #  File src/library/graphics/R/stripchart.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2013 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -55,7 +57,8 @@ function(x, method="overplot", jitter=0.1, offset=1/3, vertical=FALSE,
 	    xlim <- if(vertical) glim else dlim
 	if(is.null(ylim))
 	    ylim <- if(vertical) dlim else glim
-	plot(xlim, ylim, type="n", ann=FALSE, axes=FALSE, log=log, ...)
+	plot(xlim, ylim, xlim = xlim, ylim = ylim,
+             type = "n", ann = FALSE, axes = FALSE, log = log, ...)
 	if (frame.plot) box()
 	if(vertical) {
 	    if (axes) {
@@ -112,7 +115,7 @@ stripchart.formula <-
     m$x <- NULL
     m$na.action <- na.action # force use of default for this method
     require(stats, quietly = TRUE)
-    m[[1L]] <- as.name("model.frame")
+    m[[1L]] <- quote(stats::model.frame)
     mf <- eval(m, parent.frame())
     response <- attr(attr(mf, "terms"), "response")
     if (is.null(dlab)) dlab <- names(mf)[response]
