@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2012  The R Core Team
+ *  Copyright (C) 1997--2014  The R Core Team
  *  Copyright (C) 2002--2011  The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -35,6 +35,14 @@
 #include <GraphicsBase.h>       /* setBaseDevice */
 #include <Rmath.h>		/* eg. fmax2() */
 
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#undef _
+#define _(String) dgettext ("grDevices", String)
+#else
+#define _(String) (String)
+#endif
+
 /*--->> Documentation now in  ../include/Rgraphics.h  "API" ----- */
 
 double R_Log10(double x)
@@ -1762,6 +1770,11 @@ pGEDevDesc GNewPlot(Rboolean recording)
     /* we can call par(mfg) before any plotting.
        That sets new = TRUE and also sets currentFigure <= lastFigure
        so treat separately. */
+
+    /* The logic for when to start a new page is mimiced in the
+     * read-only par("page") in par.c, SO if you make changes
+     * to the logic here, you will need to change that as well
+     */
     if (!gpptr(dd)->new) {
 	R_GE_gcontext gc;
 	gcontextFromGP(&gc, dd);
@@ -2525,13 +2538,43 @@ void GLine(double x1, double y1, double x2, double y2, int coords, pGEDevDesc dd
 	GELine(x1, y1, x2, y2, &gc, dd);
 }
 
+/* We need extra graphics device closure handling
+   when inside a call to locator (it should raise
+   an error and return).  PR#15253
+
+   This assume that locator is running on only one device at a time, 
+   which is currently safe.
+*/
+static void (*old_close)(pDevDesc) = NULL;
+
+static void locator_close(pDevDesc dd)
+{
+    if(old_close) old_close(dd);
+    dd->close = old_close;
+    old_close = NULL;
+    error(_("graphics device closed during call to locator or identify"));
+}
+
+
 /* Read the current "pen" position. */
 Rboolean GLocator(double *x, double *y, int coords, pGEDevDesc dd)
 {
-    if(dd->dev->locator && dd->dev->locator(x, y, dd->dev)) {
-	GConvert(x, y, DEVICE, coords, dd);
-	return TRUE;
-    } else return FALSE;
+  Rboolean ret;
+  /* store original close handler (it will still be called on
+     closure) and assign new handler that throws an error
+  */
+  old_close = (dd->dev)->close;
+  dd->dev->close = &locator_close;
+  
+  if(dd->dev->locator && dd->dev->locator(x, y, dd->dev)) {
+      GConvert(x, y, DEVICE, coords, dd);
+      ret =  TRUE;
+  } else ret =  FALSE;
+  /* restore original close handler */
+  dd->dev->close = old_close;
+  old_close = NULL;
+  return ret;
+  
 }
 
 /* Access character font metric information.  */
