@@ -1,6 +1,8 @@
 #  File src/library/grid/R/roundrect.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -44,7 +46,7 @@ validDetails.roundrect <- function(x) {
         !is.unit(x$height))
         stop("'x', 'y', 'width', and 'height' must be units")
     if (!is.unit(x$r))
-        stop("'r' must be a unit object")
+        stop("'r' must be a 'unit' object")
     valid.just(x$just)
     # Make sure that x and y are of length 1
     if (length(x$x) != 1 | length(x$y) != 1 |
@@ -53,13 +55,15 @@ validDetails.roundrect <- function(x) {
     x
 }
 
-preDrawDetails.roundrect <- function(x) {
-  pushViewport(viewport(x$x, x$y, x$width, x$height, just=x$just),
-               recording=FALSE)
-}
-
-postDrawDetails.roundrect <- function(x) {
-  popViewport(recording=FALSE)
+makeContext.roundrect <- function(x) {
+    rrvp <- viewport(x$x, x$y, x$width, x$height, just=x$just,
+                     name="rrvp")
+    if (!is.null(x$vp)) {
+        x$vp <- vpStack(x$vp, rrvp)
+    } else {
+        x$vp <- rrvp
+    }
+    x
 }
 
 # x, y, is the real corner
@@ -112,15 +116,15 @@ rrpoints <- function(x) {
   list(x=xx, y=yy)
 }
 
-drawDetails.roundrect <- function(x, recording) {
+makeContent.roundrect <- function(x) {
     boundary <- rrpoints(x)
-    grid.Call.graphics("L_polygon", boundary$x, boundary$y,
-                       list(as.integer(seq_along(boundary$x))))
+    polygonGrob(boundary$x, boundary$y,
+                name=x$name, gp=x$gp, vp=x$vp)
 }
 
 xDetails.roundrect <- function(x, theta) {
     boundary <- rrpoints(x)
-    bounds <- grid.Call("L_locnBounds", boundary$x, boundary$y, theta)
+    bounds <- grid.Call(L_locnBounds, boundary$x, boundary$y, theta)
     if (is.null(bounds))
         unit(0.5, "npc")
     else
@@ -129,7 +133,7 @@ xDetails.roundrect <- function(x, theta) {
 
 yDetails.roundrect <- function(x, theta) {
     boundary <- rrpoints(x)
-    bounds <- grid.Call("L_locnBounds", boundary$x, boundary$y, theta)
+    bounds <- grid.Call(L_locnBounds, boundary$x, boundary$y, theta)
     if (is.null(bounds))
         unit(0.5, "npc")
     else
@@ -138,7 +142,7 @@ yDetails.roundrect <- function(x, theta) {
 
 widthDetails.roundrect <- function(x) {
     boundary <- rrpoints(x)
-    bounds <- grid.Call("L_locnBounds", boundary$x, boundary$y, 0)
+    bounds <- grid.Call(L_locnBounds, boundary$x, boundary$y, 0)
     if (is.null(bounds))
         unit(0, "inches")
     else
@@ -147,7 +151,7 @@ widthDetails.roundrect <- function(x) {
 
 heightDetails.roundrect <- function(x) {
     boundary <- rrpoints(x)
-    bounds <- grid.Call("L_locnBounds", boundary$x, boundary$y, 0)
+    bounds <- grid.Call(L_locnBounds, boundary$x, boundary$y, 0)
     if (is.null(bounds))
         unit(0, "inches")
     else
