@@ -1,6 +1,8 @@
 #  File src/library/grid/R/unit.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2013 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -58,8 +60,8 @@ convertUnit <- function(x, unitTo, axisFrom="x", typeFrom="location",
   if (!is.unit(x))
     stop("'x' argument must be a unit object")
   if (is.na(whatfrom) || is.na(whatto))
-    stop("Invalid 'axis' or 'type'")
-  value <- grid.Call("L_convert", x, as.integer(whatfrom),
+    stop("invalid 'axis' or 'type'")
+  value <- grid.Call(L_convert, x, as.integer(whatfrom),
                  as.integer(whatto), valid.units(unitTo))
   if (!valueOnly)
     unit(value, unitTo)
@@ -113,6 +115,15 @@ convertNative <- function(unit, dimension="x", type="location") {
               valueOnly=TRUE)
 }
 
+# This is like the "convert" functions:  it evaluates units (immediately)
+# in the current context
+calcStringMetric <- function(text) {
+    # .Call rather than .Call.graphics because it is a one-off calculation
+    metric <- grid.Call(L_stringMetric, text)
+    names(metric) <- c("ascent", "descent", "width")
+    metric
+}
+
 # NOTE: the order of the strings in these conversion functions must
 # match the order of the enums in ../src/grid.h
 # AND in ../src/unit.c (see UnitTable)
@@ -122,17 +133,19 @@ convertNative <- function(unit, dimension="x", type="location") {
                      "points", "picas", "bigpts",
                      "dida", "cicero", "scaledpts",
                      "strwidth", "strheight",
+                     "strascent", "strdescent",
                      "vplayoutwidth", "vplayoutheight", "char",
                      "grobx", "groby", "grobwidth", "grobheight",
+                     "grobascent", "grobdescent",
                      "mylines", "mychar", "mystrwidth", "mystrheight")
 
 stringUnit <- function(unit) {
-    unit == "strwidth" | unit == "strheight"
+    unit %in% c("strwidth", "strheight", "strascent", "strdescent")
 }
 
 grobUnit <- function(unit) {
-    unit == "grobwidth" | unit == "grobheight" |
-    unit == "grobx" | unit == "groby"
+    unit %in% c("grobx", "groby", "grobwidth", "grobheight",
+                "grobascent", "grobdescent")
 }
 
 dataUnit <- function(unit) {
@@ -176,7 +189,7 @@ valid.data <- function(units, data) {
         for (i in (1L:n)[str.units])
             if (!(length(data) >= i &&
                   (is.character(data[[i]]) || is.language(data[[i]]))))
-                stop("No string supplied for 'strwidth/height' unit")
+                stop("no string supplied for 'strwidth/height' unit")
     # Make sure that a grob has been specified
     grob.units <- grobUnit(units)
     if (any(grob.units))
@@ -184,7 +197,7 @@ valid.data <- function(units, data) {
             if (!(length(data) >= i &&
                   (is.grob(data[[i]]) || inherits(data[[i]], "gPath") ||
                    is.character(data[[i]]))))
-                stop("No 'grob' supplied for 'grobwidth/height' unit")
+                stop("no 'grob' supplied for 'grobwidth/height' unit")
             if (is.character(data[[i]]))
                 data[[i]] <- gPathDirect(data[[i]])
             if (inherits(data[[i]], "gPath"))
@@ -193,7 +206,7 @@ valid.data <- function(units, data) {
         }
     # Make sure that where no data is required, the data is NULL
     if (!all(sapply(data[!(str.units | grob.units)], is.null)))
-        stop("Non-NULL value supplied for plain unit")
+        stop("non-NULL value supplied for plain unit")
     data
 }
 
@@ -203,7 +216,7 @@ valid.units <- function(units) {
 
 as.character.unit <- function(x, ...) {
   class(x) <- NULL
-  paste(x, attr(x, "unit"), sep="")
+  paste0(x, attr(x, "unit"))
 }
 
 #########################
@@ -219,32 +232,32 @@ unit.arithmetic <- function(func.name, arg1, arg2=NULL) {
 Ops.unit <- function(e1, e2) {
   ok <- switch(.Generic, "+"=TRUE, "-"=TRUE, "*"=TRUE, FALSE)
   if (!ok)
-    stop(gettextf("Operator '%s' not meaningful for units", .Generic),
+    stop(gettextf("operator '%s' not meaningful for units", .Generic),
          domain = NA)
   if (.Generic == "*")
     # can only multiply a unit by a scalar
     if (nzchar(.Method[1L])) {
       if (nzchar(.Method[2L]))
-        stop("Only one operand may be a unit")
+        stop("only one operand may be a unit")
       else if (is.numeric(e2))
         # NOTE that we always put the scalar first
         # Use as.numeric to force e2 to be REAL
         unit.arithmetic(.Generic, as.numeric(e2), e1)
       else
-        stop("Non-unit operand must be numeric")
+        stop("non-unit operand must be numeric")
     } else {
       if (is.numeric(e1))
         # Use as.numeric to force e1 to be REAL
         unit.arithmetic(.Generic, as.numeric(e1), e2)
       else
-        stop("Non-unit operand must be numeric")
+        stop("non-unit operand must be numeric")
     }
   else
     # Check that both arguments are units
     if (nzchar(.Method[1L]) && nzchar(.Method[2L]))
       unit.arithmetic(.Generic, e1, e2)
     else
-      stop("Both operands must be units")
+      stop("both operands must be units")
 }
 
 ## <FIXME>
@@ -272,9 +285,9 @@ as.character.unit.arithmetic <- function(x, ...) {
   # this will recurse.
   fname <- x$fname
   if (fname == "+" || fname == "-" || fname == "*")
-    paste(x$arg1, fname, x$arg2, sep="")
+    paste0(x$arg1, fname, x$arg2)
   else
-    paste(fname, "(", paste(x$arg1, collapse=", "), ")", sep="")
+    paste0(fname, "(", paste(x$arg1, collapse=", "), ")")
 }
 
 unit.pmax <- function(...) {
@@ -286,7 +299,7 @@ unit.pmax <- function(...) {
   x <- list(...)
   numargs <- length(x)
   if (numargs == 0L)
-    stop("Zero arguments where at least one expected")
+    stop("no arguments where at least one expected")
   # how long will the result be?
   maxlength <- 0L
   for (i in seq_len(numargs))
@@ -364,7 +377,7 @@ is.unit <- function(unit) {
 }
 
 print.unit <- function(x, ...) {
-  print(as.character(x), quote=FALSE)
+  print(as.character(x), quote=FALSE, ...)
   invisible(x)
 }
 
@@ -386,12 +399,12 @@ print.unit <- function(x, ...) {
   # Allow for negative integer index
   if (any(index < 0)) {
       if (any(index > 0))
-          stop("Cannot mix signs of indices")
+          stop("cannot mix signs of indices")
       else
           index <- (1L:this.length)[index]
   }
   if (top && any(index > this.length))
-    stop("Index out of bounds (unit subsetting)")
+    stop("index out of bounds ('unit' subsetting)")
   cl <- class(x)
   units <- attr(x, "unit")
   valid.units <- attr(x, "valid.unit")
@@ -418,12 +431,12 @@ print.unit <- function(x, ...) {
   # Allow for negative integer index
   if (any(index < 0)) {
       if (any(index > 0))
-          stop("Cannot mix signs of indices")
+          stop("cannot mix signs of indices")
       else
           index <- (1L:this.length)[index]
   }
   if (top && any(index > this.length))
-    stop("Index out of bounds (unit arithmetic subsetting)")
+    stop("index out of bounds (unit arithmetic subsetting)")
 
   repSummaryUnit <- function(x, n) {
       newUnits <- lapply(seq_len(n), function(z) { get(x$fname)(x$arg1) })
@@ -451,12 +464,12 @@ print.unit <- function(x, ...) {
   # Allow for negative integer index
   if (any(index < 0)) {
       if (any(index > 0))
-          stop("Cannot mix signs of indices")
+          stop("cannot mix signs of indices")
       else
           index <- (1L:this.length)[index]
   }
   if (top && any(index > this.length))
-    stop("Index out of bounds (unit list subsetting)")
+    stop("index out of bounds (unit list subsetting)")
   cl <- class(x)
   result <- unclass(x)[(index - 1) %% this.length + 1]
   class(result) <- cl
@@ -465,6 +478,19 @@ print.unit <- function(x, ...) {
 
 # Write `[<-.unit` methods too ??
 
+#########################
+# str() method
+#########################
+
+# Should work fine on atomic units and on unit.list
+# The problem arises with unit.arithmetic, which are stored as lists
+# but act like vectors
+# (e.g., report length greater than number of list components)
+str.unit.arithmetic <- function(object, ...) {
+    cat("Class 'unit.arithmetic' [1:", length(object), "] ", sep="")
+    str(unclass(object), ...)
+}
+
 #########################
 # "c"ombining unit objects
 #########################
@@ -483,7 +509,7 @@ print.unit <- function(x, ...) {
 unit.c <- function(...) {
     x <- list(...)
     if (!all(sapply(x, is.unit)))
-        stop("It is invalid to combine unit objects with other types")
+        stop("it is invalid to combine 'unit' objects with other types")
     listUnit <- function(x) {
         inherits(x, "unit.list") ||
         inherits(x, "unit.arithmetic")
@@ -521,7 +547,7 @@ unit.list.from.list <- function(x) {
 
 rep.unit <- function(x, times=1, length.out=NA, each=1, ...) {
     if (length(x) == 0)
-        stop("Invalid unit object")
+        stop("invalid 'unit' object")
 
     # Determine an approprite index, then call subsetting code
     repIndex <- rep(seq_along(x), times=times, length.out=length.out, each=each)
@@ -531,7 +557,7 @@ rep.unit <- function(x, times=1, length.out=NA, each=1, ...) {
 # Vestige from when rep() was not generic
 unit.rep <- function (x, ...)
 {
-  warning("unit.rep has been deprecated in favour of a unit method for the generic rep function")
+  warning("'unit.rep' has been deprecated in favour of a unit method for the generic rep function", domain = NA)
   rep(x, ...)
 }
 
@@ -552,14 +578,14 @@ length.unit.arithmetic <- function(x) {
          "+"=max(length(x$arg1), length(x$arg2)),
          "-"=max(length(x$arg1), length(x$arg2)),
          "*"=max(length(x$arg1), length(x$arg2)),
-         "min"=1,
-         "max"=1,
-         "sum"=1)
+         "min" = 1L,
+         "max" = 1L,
+         "sum" = 1L)
 }
 
 # Vestige of when length was not generic
 unit.length <- function(unit) {
-   warning("unit.length has been deprecated in favour of a unit method for the generic length function")
+   warning("'unit.length' has been deprecated in favour of a unit method for the generic length function", domain = NA)
    length(unit)
 }
 
@@ -591,6 +617,30 @@ stringHeight <- function(string) {
     unit(rep(1, n), "strheight", data=data)
 }
 
+stringAscent <- function(string) {
+    n <- length(string)
+    if (is.language(string)) {
+        data <- vector("list", n)
+        for (i in 1L:n)
+            data[[i]] <- string[i]
+    } else {
+        data <- as.list(as.character(string))
+    }
+    unit(rep(1, n), "strascent", data=data)
+}
+
+stringDescent <- function(string) {
+    n <- length(string)
+    if (is.language(string)) {
+        data <- vector("list", n)
+        for (i in 1L:n)
+            data[[i]] <- string[i]
+    } else {
+        data <- as.list(as.character(string))
+    }
+    unit(rep(1, n), "strdescent", data=data)
+}
+
 convertTheta <- function(theta) {
     if (is.character(theta))
         # Allow some aliases for common angles
@@ -599,7 +649,7 @@ convertTheta <- function(theta) {
                north=90,
                west=180,
                south=270,
-               stop("Invalid theta"))
+               stop("invalid 'theta'"))
     else
         # Ensure theta in [0, 360)
         theta <- as.numeric(theta) %% 360
@@ -689,6 +739,48 @@ grobHeight.default <- function(x) {
   unit(1, "grobheight", data=gPathDirect(as.character(x)))
 }
 
+# grobAscent
+grobAscent <- function(x) {
+  UseMethod("grobAscent")
+}
+
+grobAscent.grob <- function(x) {
+  unit(1, "grobascent", data=x)
+}
+
+grobAscent.gList <- function(x) {
+  unit(rep(1, length(gList)), "grobascent", data=x)
+}
+
+grobAscent.gPath <- function(x) {
+  unit(1, "grobascent", data=x)
+}
+
+grobAscent.default <- function(x) {
+  unit(1, "grobascent", data=gPathDirect(as.character(x)))
+}
+
+# grobDescent
+grobDescent <- function(x) {
+  UseMethod("grobDescent")
+}
+
+grobDescent.grob <- function(x) {
+  unit(1, "grobdescent", data=x)
+}
+
+grobDescent.gList <- function(x) {
+  unit(rep(1, length(gList)), "grobdescent", data=x)
+}
+
+grobDescent.gPath <- function(x) {
+  unit(1, "grobdescent", data=x)
+}
+
+grobDescent.default <- function(x) {
+  unit(1, "grobdescent", data=gPathDirect(as.character(x)))
+}
+
 #########################
 # Function to decide which values in a unit are "absolute" (do not depend
 # on parent's drawing context or size)
@@ -700,7 +792,7 @@ absolute <- function(unit) {
                c("cm", "inches", "lines", "null",
                  "mm", "points", "picas", "bigpts",
                  "dida", "cicero", "scaledpts",
-                 "strwidth", "strheight", "char",
+                 "strwidth", "strheight", "strascent", "strdescent", "char",
                  "mylines", "mychar", "mystrwidth", "mystrheight")))
 }
 
