@@ -1,6 +1,8 @@
 #  File src/library/methods/R/makeBasicFunsList.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2013 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -20,6 +22,8 @@
 ## uses the primitive list and the function .addBasicGeneric
 ## defined (earlier) in BasicFunsList.R
 
+utils::globalVariables(".addBasicGeneric")
+
 .makeBasicFuns <- function(where)
 {
     funs <- get(".BasicFunsList", envir=where)
@@ -177,6 +181,20 @@
 	       signature = c("x", "norm"), where = where)
     setGenericImplicit("rcond", where, FALSE)
 
+    setGeneric("norm", function(x, type, ...) standardGeneric("norm"),
+	       useAsDefault = function(x, type, ...) base::norm(x, type, ...),
+	       signature = c("x", "type"), where = where)
+    setGenericImplicit("norm", where, FALSE)
+
+    setGeneric("backsolve", function(r, x, k = ncol(r), upper.tri = TRUE, transpose = FALSE, ...)
+	       standardGeneric("backsolve"),
+	       useAsDefault =
+	       function(r, x, k = ncol(r), upper.tri = TRUE, transpose = FALSE, ...)
+	       base::backsolve(r, x, k = k,
+			       upper.tri = upper.tri, transpose = transpose, ...),
+	       signature = c("r", "x"), where = where)
+    setGenericImplicit("backsolve", where, FALSE)
+
     setGeneric("colMeans", function(x, na.rm = FALSE, dims = 1, ...)
 			standardGeneric("colMeans"),
 	       useAsDefault = function(x, na.rm = FALSE, dims = 1, ...)
@@ -202,6 +220,13 @@
     setGenericImplicit("rowMeans", where, FALSE)
     setGenericImplicit("rowSums",  where, FALSE)
 
+    setGeneric("sample", function(x, size, replace = FALSE, prob = NULL, ...)
+			standardGeneric("sample"),
+	       useAsDefault = function(x, size, replace = FALSE, prob = NULL, ...)
+			base::sample(x, size, replace=replace, prob=prob, ...),
+	       signature = c("x", "size"), where = where)
+    setGenericImplicit("sample", where, FALSE)
+
     ## not implicitGeneric() which is not yet available "here"
     registerImplicitGenerics(where = where)
 }
