@@ -1,6 +1,8 @@
 #  File src/library/stats/R/HoltWinters.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 2002-2013 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -14,6 +16,8 @@
 #  A copy of the GNU General Public License is available at
 #  http://www.r-project.org/Licenses/
 
+# Originally contributed by David Meyer
+
 HoltWinters <-
 function (x,
 
@@ -39,15 +43,15 @@ function (x,
     f <- frequency(x)
 
     if(!is.null(alpha) && (alpha == 0))
-        stop ("cannot fit models without level ('alpha' must not be 0 or FALSE).")
+        stop ("cannot fit models without level ('alpha' must not be 0 or FALSE)")
     if(!all(is.null(c(alpha, beta, gamma))) &&
         any(c(alpha, beta, gamma) < 0 || c(alpha, beta, gamma) > 1))
-        stop ("'alpha', 'beta' and 'gamma' must be within the unit interval.")
+        stop ("'alpha', 'beta' and 'gamma' must be within the unit interval")
     if((is.null(gamma) || gamma > 0)) {
         if (seasonal == "multiplicative" && any(x == 0))
-            stop ("data must be non-zero for multiplicative Holt-Winters.")
+            stop ("data must be non-zero for multiplicative Holt-Winters")
         if (start.periods < 2)
-            stop ("need at least 2 periods to compute seasonal start values.")
+            stop ("need at least 2 periods to compute seasonal start values")
     }
 
     ## initialization
@@ -80,15 +84,19 @@ function (x,
     }
 
     ## Call to filtering loop
-    len <- length(x) - start.time + 1
+    lenx <- as.integer(length(x))
+    if (is.na(lenx)) stop("invalid length(x)")
+
+    len <- lenx - start.time + 1
     hw <- function(alpha, beta, gamma)
         .C(C_HoltWinters,
            as.double(x),
-           as.integer(length(x)),
-           as.double(max(min(alpha,1),0)),
-           as.double(max(min(beta,1),0)),
-           as.double(max(min(gamma,1),0)),
+           lenx,
+           as.double(max(min(alpha, 1), 0)),
+           as.double(max(min(beta, 1), 0)),
+           as.double(max(min(gamma, 1), 0)),
            as.integer(start.time),
+           ## no idea why this is so: same as seasonal != "multiplicative"
            as.integer(! + (seasonal == "multiplicative")),
            as.integer(f),
            as.integer(!is.logical(beta) || beta),
@@ -102,9 +110,7 @@ function (x,
            SSE = as.double(0),
            level = double(len + 1L),
            trend = double(len + 1L),
-           seasonal = double(len + f),
-
-	   PACKAGE = "stats"
+           seasonal = double(len + f)
            )
 
     ## if alpha and/or beta and/or gamma are omitted, use optim to find the
@@ -120,8 +126,12 @@ function (x,
                 sol   <- optim(optim.start, error, method = "L-BFGS-B",
                                lower = c(0, 0, 0), upper = c(1, 1, 1),
                                control = optim.control)
-                if(sol$convergence || any(sol$par < 0 | sol$par > 1))
-                    stop("optimization failure")
+                if(sol$convergence || any(sol$par < 0 | sol$par > 1)) {
+                    if (sol$convergence > 50) {
+                        warning(gettextf("optimization difficulties: %s",
+                                         sol$message), domain = NA)
+                    } else stop("optimization failure")
+                }
                 alpha <- sol$par[1L]
                 beta  <- sol$par[2L]
                 gamma <- sol$par[3L]
@@ -133,8 +143,12 @@ function (x,
                                error, method = "L-BFGS-B",
                                lower = c(0, 0), upper = c(1, 1),
                                control = optim.control)
-                if(sol$convergence || any(sol$par < 0 | sol$par > 1))
-                    stop("optimization failure")
+                if(sol$convergence || any(sol$par < 0 | sol$par > 1)) {
+                    if (sol$convergence > 50) {
+                        warning(gettextf("optimization difficulties: %s",
+                                         sol$message), domain = NA)
+                    } else stop("optimization failure")
+                }
                 alpha <- sol$par[1L]
                 gamma <- sol$par[2L]
             }
@@ -148,8 +162,12 @@ function (x,
                                error, method = "L-BFGS-B",
                                lower = c(0, 0), upper = c(1, 1),
                                control = optim.control)
-                if(sol$convergence || any(sol$par < 0 | sol$par > 1))
-                    stop("optimization failure")
+                if(sol$convergence || any(sol$par < 0 | sol$par > 1)) {
+                    if (sol$convergence > 50) {
+                        warning(gettextf("optimization difficulties: %s",
+                                         sol$message), domain = NA)
+                    } else stop("optimization failure")
+                }
                 beta  <- sol$par[1L]
                 gamma <- sol$par[2L]
             } else {
@@ -171,8 +189,12 @@ function (x,
                                error, method = "L-BFGS-B",
                                lower = c(0, 0), upper = c(1, 1),
                                control = optim.control)
-                if(sol$convergence || any(sol$par < 0 | sol$par > 1))
-                    stop("optimization failure")
+                if(sol$convergence || any(sol$par < 0 | sol$par > 1)) {
+                    if (sol$convergence > 50) {
+                        warning(gettextf("optimization difficulties: %s",
+                                         sol$message), domain = NA)
+                    } else stop("optimization failure")
+                }
                 alpha <- sol$par[1L]
                 beta  <- sol$par[2L]
             } else {
@@ -229,7 +251,7 @@ function (x,
 
 ## Predictions, optionally with prediction intervals
 predict.HoltWinters <-
-    function (object, n.ahead = 1, prediction.interval = FALSE,
+    function (object, n.ahead = 1L, prediction.interval = FALSE,
               level = 0.95, ...)
 {
     f <- frequency(object$x)
@@ -248,7 +270,7 @@ predict.HoltWinters <-
 
     ## compute predictions
     # level
-    fit <- rep(as.vector(object$coefficients[1L]),n.ahead)
+    fit <- rep(as.vector(object$coefficients[1L]) ,n.ahead)
     # trend
     if (!is.logical(object$beta) || object$beta)
         fit <- fit + as.vector((1L:n.ahead)*object$coefficients[2L])
@@ -320,13 +342,13 @@ print.HoltWinters <- function (x, ...)
     cat("Holt-Winters exponential smoothing",
         if (is.logical(x$beta) && !x$beta) "without" else "with", "trend and",
         if (is.logical(x$gamma) && !x$gamma) "without" else
-        paste(if (is.logical(x$beta) && !x$beta) "with ", x$seasonal, sep=""),
-        "seasonal component.\n")
-    cat("\nCall:\n", deparse (x$call), "\n\n")
+        paste0(if (is.logical(x$beta) && !x$beta) "with ", x$seasonal),
+        "seasonal component.")
+    cat("\n\nCall:\n", deparse (x$call), "\n\n", sep = "")
     cat("Smoothing parameters:\n")
-    cat(" alpha: ", x$alpha, "\n")
-    cat(" beta : ", x$beta, "\n")
-    cat(" gamma: ", x$gamma, "\n\n")
+    cat(" alpha: ", x$alpha, "\n", sep = "")
+    cat(" beta : ", x$beta, "\n", sep = "")
+    cat(" gamma: ", x$gamma, "\n\n", sep = "")
 
     cat("Coefficients:\n")
     print(t(t(x$coefficients)))
