@@ -1,7 +1,7 @@
 #  File src/library/stats/R/aggregate.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -50,19 +50,28 @@ function(x, by, FUN, ..., simplify = TRUE)
         ind <- which(!nzchar(nam))
         names(by)[ind] <- paste("Group", ind, sep = ".")
     }
-
-    ## Similar to interaction(drop = TRUE), but using integer arithmetic
-    ## is definitely a bad idea for boundary cases ...
+ 
     nrx <- NROW(x)
-    grp <- double(nrx)
-    for(ind in rev(by)) {
-        if(length(ind) != nrx)
-            stop("arguments must have same length")
-        ind <- as.factor(ind)
-        grp <- grp * nlevels(ind) + (as.integer(ind) - 1L)
-    }
 
+    if(any(unlist(lapply(by, length)) != nrx))
+        stop("arguments must have same length")
+	
     y <- as.data.frame(by, stringsAsFactors = FALSE)
+    keep <- complete.cases(by)
+    y <- y[keep, , drop = FALSE]
+    x <- x[keep, , drop = FALSE]
+    
+    nrx <- NROW(x)
+
+    # Generate a group identifier vector with integers and dots.
+    ident <- function(x){
+           y <- as.integer(as.factor(x))
+           z <- gsub(" ", "0", format(y, scientific = FALSE)) # for right sort order
+           return(z)
+    }
+    grp <- rank(do.call(paste, c(lapply(rev(y), ident), list(sep = "."))), 
+                ties.method = "min")
+
     y <- y[match(sort(unique(grp)), grp, 0L), , drop = FALSE]
     nry <- NROW(y)
     z <- lapply(x,
@@ -116,7 +125,7 @@ function(formula, data, FUN, ..., subset, na.action = na.omit)
     if(is.matrix(eval(m$data, parent.frame())))
         m$data <- as.data.frame(data)
     m$... <- m$FUN <- NULL
-    m[[1L]] <- as.name("model.frame")
+    m[[1L]] <- quote(stats::model.frame)
 
     if (formula[[2L]] == ".") {
         ## LHS is a dot, expand it ...
