@@ -2,7 +2,7 @@
 #  Part of the R package, http://www.R-project.org
 #
 #  Copyright (C) 1998 B. D. Ripley
-#  Copyright (C) 1998-2012 The R Core Team
+#  Copyright (C) 1998-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -31,14 +31,12 @@ dummy.coef.lm <- function(object, use.na=FALSE, ...)
     if(!length(xl)) {			# no factors in model
 	return(as.list(coef(object)))
     }
-    nxl <- rep.int(1, length(vars))
-    names(nxl) <- vars
-    tmp <- unlist(lapply(xl, length))
+    nxl <- setNames(rep.int(1, length(vars)), vars)
+    tmp <- unlist(lapply(xl, length)) ## ?? vapply(xl, length, 1L)
     nxl[names(tmp)] <- tmp
     lterms <- apply(facs, 2L, function(x) prod(nxl[x > 0]))
     nl <- sum(lterms)
-    args <- vector("list", length(vars))
-    names(args) <- vars
+    args <- setNames(vector("list", length(vars)), vars)
     for(i in vars)
 	args[[i]] <- if(nxl[[i]] == 1) rep.int(1, nl)
 	else factor(rep.int(xl[[i]][1L], nl), levels = xl[[i]])
@@ -73,17 +71,15 @@ dummy.coef.lm <- function(object, use.na=FALSE, ...)
     coef <- object$coefficients
     if(!use.na) coef[is.na(coef)] <- 0
     asgn <- attr(mm,"assign")
-    res <- vector("list", length(tl))
-    names(res) <- tl
+    res <- setNames(vector("list", length(tl)), tl)
     for(j in seq_along(tl)) {
 	keep <- asgn == j
-	ans <- drop(mm[rn == tl[j], keep, drop=FALSE] %*% coef[keep])
-	names(ans) <- rnn[rn == tl[j]]
-	res[[j]] <- ans
+	ij <- rn == tl[j]
+	res[[j]] <-
+	    setNames(drop(mm[ij, keep, drop=FALSE] %*% coef[keep]), rnn[ij])
     }
     if(int > 0) {
-	res <- c(list(coef[int]), res)
-	names(res)[1L] <- "(Intercept)"
+	res <- c(list("(Intercept)" = coef[int]), res)
     }
     class(res) <- "dummy_coef"
     res
@@ -101,14 +97,12 @@ dummy.coef.aovlist <- function(object, use.na = FALSE, ...)
     if(!length(xl)) {			# no factors in model
 	return(as.list(coef(object)))
     }
-    nxl <- rep.int(1, length(vars))
-    names(nxl) <- vars
+    nxl <- setNames(rep.int(1, length(vars)), vars)
     tmp <- unlist(lapply(xl, length))
     nxl[names(tmp)] <- tmp
     lterms <- apply(facs, 2L, function(x) prod(nxl[x > 0]))
     nl <- sum(lterms)
-    args <- vector("list", length(vars))
-    names(args) <- vars
+    args <- setNames(vector("list", length(vars)), vars)
     for(i in vars)
 	args[[i]] <- if(nxl[[i]] == 1) rep.int(1, nl)
 	else factor(rep.int(xl[[i]][1L], nl), levels = xl[[i]])
@@ -136,9 +130,8 @@ dummy.coef.aovlist <- function(object, use.na = FALSE, ...)
     if (!int) form <- paste(form, "- 1")
     mm <- model.matrix(terms(formula(form)), dummy,
 		       attr(object, "contrasts"), xl)
-    res <- vector("list", length(object))
-    names(res) <- names(object)
     tl <- c("(Intercept)", tl)
+    res <- setNames(vector("list", length(object)), names(object))
     allasgn <- attr(mm, "assign")
     for(i in names(object)) {
 	coef <- object[[i]]$coefficients
@@ -146,17 +139,17 @@ dummy.coef.aovlist <- function(object, use.na = FALSE, ...)
 	asgn <- object[[i]]$assign
 	uasgn <- unique(asgn)
 	tll <- tl[1 + uasgn]
-	mod <- vector("list", length(tll))
-	names(mod) <- tll
+	mod <- setNames(vector("list", length(tll)), tll)
 	for(j in uasgn) {
-	    if(j == 0) {
-		ans <- structure(coef[asgn == j], names="(Intercept)")
-	    } else {
-		ans <- drop(mm[rn == tl[1+j], allasgn == j, drop=FALSE] %*%
-			    coef[asgn == j])
-		names(ans) <- rnn[rn == tl[1+j]]
-	    }
-	    mod[[tl[1+j]]] <- ans
+	    mod[[tl[1+j]]] <-
+		if(j == 0) {
+		    structure(coef[asgn == j], names="(Intercept)")
+		} else {
+		    ij <- rn == tl[1+j]
+		    setNames(drop(mm[ij, allasgn == j, drop=FALSE] %*%
+				  coef[asgn == j]),
+			     rnn[ij])
+		}
 	}
 	res[[i]] <- mod
     }
@@ -168,11 +161,11 @@ print.dummy_coef <- function(x, ..., title)
 {
     terms <- names(x)
     n <- length(x)
-    nm <- max(sapply(x, length))
+    nm <- max(vapply(x, length, 1L))
     ans <- matrix("", 2L*n, nm)
     rn <- rep.int("", 2L*n)
     line <- 0
-    for (j in seq(n)) {
+    for (j in seq_len(n)) {
 	this <- x[[j]]
 	n1 <- length(this)
 	if(n1 > 1) {
