@@ -1,6 +1,8 @@
 #  File src/library/stats/R/interaction.plot.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2013 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -16,7 +18,7 @@
 
 interaction.plot <-
     function(x.factor, trace.factor, response, fun=mean,
-             type = c("l", "p", "b"), legend = TRUE,
+	     type = c("l", "p", "b", "o", "c"), legend = TRUE,
              trace.label=deparse(substitute(trace.factor)), fixed=FALSE,
              xlab = deparse(substitute(x.factor)), ylab = ylabel,
              ylim = range(cells, na.rm=TRUE),
@@ -46,6 +48,7 @@ interaction.plot <-
     xlim <- range(xvals)
     xleg <- xlim[2L] + 0.05 * diff(xlim)
     xlim <- xlim + c(-0.2/nr, if(legend) 0.2 + 0.02*nch else 0.2/nr) * diff(xlim)
+    dev.hold(); on.exit(dev.flush())
     matplot(xvals, cells, ..., type = type, xlim = xlim, ylim = ylim,
             xlab = xlab, ylab = ylab, axes = axes, xaxt = "n",
             col = col, lty = lty, pch = pch)
@@ -63,7 +66,7 @@ interaction.plot <-
         if(!is.null(xpd) || { xpd. <- par("xpd")
                               !is.na(xpd.) && !xpd. && (xpd <- TRUE)}) {
             op <- par(xpd = xpd)
-            on.exit(par(op))
+            on.exit(par(op), add = TRUE)
         }
         text(xleg, ylim[2L] - 0.05 * yrng, paste("  ", trace.label), adj = 0)
         if(!fixed) {
