@@ -1,7 +1,7 @@
 #  File src/library/stats/R/mantelhaen.test.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -141,12 +141,8 @@ function(x, y = NULL, z = NULL,
             ## Density of the *central* product hypergeometric
             ## distribution on its support: store for once as this is
             ## needed quite a bit.
-            dc <- .C(C_d2x2xk,
-                     as.integer(K),
-                     as.double(m),
-                     as.double(n),
-                     as.double(t),
-                     d = double(hi - lo + 1))$d
+
+            dc <- .Call(C_d2x2xk, K, m, n, t, hi - lo + 1L)
             logdc <- log(dc)
 
             dn2x2xk <- function(ncp) {
