@@ -2,7 +2,7 @@
 #  Part of the R package, http://www.R-project.org
 #
 #  Copyright (C) 1999-1999 Saikat DebRoy, Douglas M. Bates, Jose C. Pinheiro
-#  Copyright (C) 2000-7    The R Core Team
+#  Copyright (C) 2000-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -562,7 +562,7 @@ nls <-
                            env = environment(formula))
             mf$start <- mf$control <- mf$algorithm <- mf$trace <- mf$model <- NULL
             mf$lower <- mf$upper <- NULL
-            mf[[1L]] <- as.name("model.frame")
+            mf[[1L]] <- quote(stats::model.frame)
             mf <- eval.parent(mf)
             n <- nrow(mf)
             mf <- as.list(mf)
@@ -598,7 +598,7 @@ nls <-
     }
     ## Iterate
     if (algorithm != "port") {
-	if (!missing(lower) || !missing(upper))
+	if (!identical(lower, -Inf) || !identical(upper, +Inf))
 	    warning('upper and lower bounds ignored unless algorithm = "port"')
         convInfo <- .Call(C_nls_iter, m, ctrl, trace)
 	nls.out <- list(m = m, convInfo = convInfo,
