@@ -1,7 +1,7 @@
 #  File src/library/stats/R/r2dtable.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -26,7 +26,5 @@ r2dtable <- function(n, r, c)
 	stop("invalid argument 'c'")
     if(sum(r) != sum(c))
 	stop("arguments 'r' and 'c' must have the same sums")
-    .Call("R_r2dtable",
-	  as.integer(n), as.integer(r), as.integer(c),
-          PACKAGE = "base")
+    .Call(C_r2dtable, as.integer(n), as.integer(r), as.integer(c))
 }
