@@ -1,7 +1,7 @@
 #  File src/library/stats/R/reshape.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
 #  http://www.r-project.org/Licenses/
 
 reshape <-
-    function(data, varying= NULL, v.names= NULL, timevar = "time", idvar = "id",
+    function(data, varying = NULL, v.names = NULL, timevar = "time",
+             idvar = "id",
              ids = 1L:NROW(data), times = seq_along(varying[[1L]]),
              drop = NULL, direction, new.row.names = NULL,
              sep = ".",
@@ -42,7 +43,7 @@ reshape <-
                         substr(nms, regexpr(re,nms) + 1L, 10000L))
 
         if (ncol(nn) != 2L)
-            stop("Failed to guess time-varying variables from their names")
+            stop("failed to guess time-varying variables from their names")
 
 
         vn <- unique(nn[,1])
@@ -102,7 +103,7 @@ reshape <-
             rval <- d
 
             if (length(times) == 1L) {
-                if (drop.idvar) rval[, idvar]<-NULL
+                if (drop.idvar) rval[, idvar] <- NULL
                 return(rval)
             }
             if (is.null(new.row.names))
@@ -123,7 +124,7 @@ reshape <-
             }
 
             ## if we created a temporary id variable, drop it
-            if (drop.idvar) rval[, idvar]<-NULL
+            if (drop.idvar) rval[, idvar] <- NULL
 
             attr(rval,"reshapeLong") <- undoInfo
             return(rval)
@@ -154,7 +155,7 @@ reshape <-
         times <- unique(data[, timevar])
         if (any(is.na(times)))
             warning("there are records with missing times, which will be dropped.")
-        undoInfo$times<-times
+        undoInfo$times <- times
 
         if (is.null(v.names))
             v.names <- names(data)[!(names(data) %in% c(timevar, idvar, orig.idvar))]
@@ -249,12 +250,12 @@ reshape <-
                    varying <- guess(varying)
                else {
                    if (length(varying) %% length(v.names))
-                       stop("length of v.names does not evenly divide length of varying")
+                       stop("length of 'v.names' does not evenly divide length of 'varying'")
                    ntimes <- length(varying) %/% length(v.names)
                    if (missing(times))
                        times <- seq_len(ntimes)
                    else if (length(times) != ntimes)
-                       stop("length of varying must be the product of length of v.names and length of times")
+                       stop("length of 'varying' must be the product of length of 'v.names' and length of 'times'")
                    varying <- split(varying, rep(v.names, ntimes))
                    attr(varying, "v.names") <- v.names
                    attr(varying, "times") <- times
