@@ -1,7 +1,7 @@
 #  File src/library/stats/R/ts.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -538,7 +538,8 @@ plot.ts <-
 	    if(do.lab)
 		text(xy, labels =
 		     if(is.character(xy.labels)) xy.labels
-		     else if(all(tsp(x) == tsp(y))) formatC(time(x), width = 1)
+		     else if(all(tsp(x) == tsp(y)))
+                         formatC(unclass(time(x)), width = 1)
 		     else seq_along(xy$x),
 		     col = col, cex = cex)
 	    if(xy.lines)
@@ -558,10 +559,10 @@ plot.ts <-
 	    k <- ncol(x)
 	    tx <- time(x)
 	    xy <- xy.coords(x = matrix(rep.int(tx, k), ncol = k),
-			    y = x, log=log)
+			    y = x, log = log)
 	    xy$x <- tx
 	}
-	else xy <- xy.coords(x, NULL, log=log)
+	else xy <- xy.coords(x, NULL, log = log)
 	if(is.null(xlim)) xlim <- range(xy$x)
 	if(is.null(ylim)) ylim <- range(xy$y[is.finite(xy$y)])
 	plot.new()
@@ -697,7 +698,7 @@ window.ts <- function (x, ...) as.ts(window.default(x, ...))
     m <- match.call(expand.dots = FALSE)
     m$value <- NULL
     m$extend <- TRUE
-    m[[1L]] <- as.name("window")
+    m[[1L]] <- quote(stats::window)
     xx <- eval.parent(m)
     xxtsp <- tsp(xx)
     start <- xxtsp[1L]; end <- xxtsp[2L]
