@@ -25,6 +25,13 @@
 #include <R_ext/Applic.h>
 
 #include "statsR.h"
+#undef _
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#define _(String) dgettext ("stats", String)
+#else
+#define _(String) (String)
+#endif
 
 SEXP getListElement(SEXP list, char *str)
 {
@@ -249,7 +256,8 @@ SEXP optim(SEXP call, SEXP op, SEXP args, SEXP rho)
 	tmax = asInteger(getListElement(options, "tmax"));
 	temp = asReal(getListElement(options, "temp"));
 	if (trace) trace = asInteger(getListElement(options, "REPORT"));
-	if (tmax == NA_INTEGER) error(_("'tmax' is not an integer"));
+	if (tmax == NA_INTEGER || tmax < 1) // PR#15194
+	    error(_("'tmax' is not a positive integer"));
 	if (!isNull(gr)) {
 	    if (!isFunction(gr)) error(_("'gr' is not a function"));
 		PROTECT(OS->R_gcall = lang2(gr, R_NilValue));
