@@ -1,7 +1,7 @@
 #  File src/library/tools/R/writePACKAGES.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -241,7 +241,7 @@ function(ap)
         wh <- which(dp == pkgs)
         vers <- package_version(ap[wh, "Version"])
         keep_ver <- max(vers)
-        keep_idx = which(vers == keep_ver)[1L] # they might all be max
+	keep_idx <- which.max(vers == keep_ver) # they might all be max
         wh <- wh[-keep_idx]
         end_i <- i + length(wh) - 1L
         stale_dups[i:end_i] <- wh
