@@ -16,5 +16,6 @@
 #  A copy of the GNU General Public License is available at
 #  http://www.r-project.org/Licenses/
 
-Rd_parse <-
-function(file, text = NULL) .Defunct("parse_Rd")
+## Defunct 2009-08-19
+## Removed for 3.0.0
+## Rd_parse <-function(file, text = NULL) .Defunct("parse_Rd")
