@@ -36,7 +36,7 @@ RweaveLatexSetup <-
     dots <- list(...)
     if (is.null(output)) {
         prefix.string <- basename(sub(syntax$extension, "", file))
-        output <- paste(prefix.string, "tex", sep=".")
+        output <- paste(prefix.string, "tex", sep = ".")
     } else prefix.string <- basename(sub("\\.tex$", "", output))
 
     if (!quiet) cat("Writing to file ", output, "\n",
@@ -109,7 +109,7 @@ makeRweaveLatexCodeRunner <- function(evalFunc = RweaveEvalWithOpt)
                            width = width, height = height,
                            res = options$resolution, units = "in")
         jpeg.Swd <- function(name, width, height, options, ...)
-            grDevices::jpeg(filename = paste(chunkprefix, "png", sep = "."),
+            grDevices::jpeg(filename = paste(chunkprefix, "jpeg", sep = "."),
                             width = width, height = height,
                             res = options$resolution, units = "in")
 
@@ -158,8 +158,12 @@ makeRweaveLatexCodeRunner <- function(evalFunc = RweaveEvalWithOpt)
                     if (!is.null(options$grdevice)) cat("", options$grdevice)
                 }
             }
+            cat(" (")
             if (!is.null(options$label))
-                cat(" (label = ", options$label, ")", sep = "")
+                cat("label = ", options$label, ", ", sep = "")
+            filenum <- attr(chunk, "srcFilenum")[1]
+            filename <- attr(chunk, "srcFilenames")[filenum]
+            cat(basename(filename), ":", attr(chunk, "srclines")[1], ")", sep = "")
             cat("\n")
         }
 
@@ -300,9 +304,10 @@ makeRweaveLatexCodeRunner <- function(evalFunc = RweaveEvalWithOpt)
             if (options$eval) {
                 tmpcon <- file()
                 sink(file = tmpcon)
-                err <- evalFunc(ce, options)
-                cat("\n")           # make sure final line is complete
-                sink()
+                err <- tryCatch(evalFunc(ce, options), finally = {
+                     cat("\n")           # make sure final line is complete
+                     sink()
+                })
                 output <- readLines(tmpcon)
                 close(tmpcon)
                 ## delete empty output
@@ -382,8 +387,7 @@ makeRweaveLatexCodeRunner <- function(evalFunc = RweaveEvalWithOpt)
         if (is.null(options$label) && options$split) close(chunkout)
 
         if (options$split && options$include) {
-            cat("\\input{", chunkprefix, "}\n", sep = "",
-                file = object$output)
+            cat("\\input{", chunkprefix, "}\n", sep = "", file = object$output)
             linesout[thisline + 1L] <- srcline
             filenumout[thisline + 1L] <- srcfilenum
             thisline <- thisline + 1L
@@ -523,22 +527,24 @@ RweaveLatexFinish <- function(object, error = FALSE)
         linesout <- object$linesout
         filenumout <- object$filenumout
         filenames <- object$srcFilenames[filenumout]
-        filegps <- rle(filenames)
-        offset <- 0L
-        for (i in seq_along(filegps$lengths)) {
-            len <- filegps$lengths[i]
-            inputname <- filegps$values[i]
-            vals <- rle(diff(linesout[offset + seq_len(len)]))
-            vals <- c(linesout[offset + 1L], as.numeric(rbind(vals$lengths, vals$values)))
-    	    concordance <- paste(strwrap(paste(vals, collapse = " ")), collapse = " %\n")
-    	    special <- paste("\\Sconcordance{concordance:", outputname, ":",
-                         inputname, ":",
-                         if (offset) paste0("ofs ", offset, ":") else "",
-                         "%\n",
-                         concordance,"}\n", sep = "")
-    	    cat(special, file = object$concordfile, append=offset > 0L)
-    	    offset <- offset + len
-    	}
+	if (!is.null(filenames)) {  # Might be NULL if an error occurred
+	    filegps <- rle(filenames)
+	    offset <- 0L
+	    for (i in seq_along(filegps$lengths)) {
+		len <- filegps$lengths[i]
+		inputname <- filegps$values[i]
+		vals <- rle(diff(linesout[offset + seq_len(len)]))
+		vals <- c(linesout[offset + 1L], as.numeric(rbind(vals$lengths, vals$values)))
+		concordance <- paste(strwrap(paste(vals, collapse = " ")), collapse = " %\n")
+		special <- paste0("\\Sconcordance{concordance:", outputname, ":",
+			     inputname, ":",
+			     if (offset) paste0("ofs ", offset, ":") else "",
+			     "%\n",
+			     concordance,"}\n")
+		cat(special, file = object$concordfile, append=offset > 0L)
+		offset <- offset + len
+	    }
+	}
     }
     invisible(outputname)
 }
@@ -605,14 +611,12 @@ RweaveChunkPrefix <- function(options)
 {
     if (!is.null(options$label)) {
         if (options$prefix)
-            chunkprefix <- paste(options$prefix.string, "-",
-                                 options$label, sep = "")
+            chunkprefix <- paste0(options$prefix.string, "-", options$label)
         else
             chunkprefix <- options$label
     } else
-        chunkprefix <- paste(options$prefix.string, "-",
-                             formatC(options$chunknr, flag = "0", width = 3),
-                             sep = "")
+        chunkprefix <- paste0(options$prefix.string, "-",
+                              formatC(options$chunknr, flag = "0", width = 3))
     chunkprefix
 }
 
@@ -728,7 +732,7 @@ RtangleRuncode <-  function(object, chunk, options)
             srclines <- attr(chunk, "srclines")
             srcfilenum <- attr(chunk, "srcFilenum")
             ## this currently includes the chunk header
-            lno <- if (length(srclines)) paste(min(srclines), max(srclines), sep="-") else srclines
+            lno <- if (length(srclines)) paste(min(srclines), max(srclines), sep = "-") else srclines
             fn <- sub('[^"]*"([^"]+).*', "\\1", lnos[1L])
         }
         cat("###################################################\n",
