@@ -1,7 +1,7 @@
 #  File src/library/utils/R/read.DIF.R
 #  Part of the R package, http://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -146,9 +146,9 @@ read.DIF <- function(file, header = FALSE, dec = ".",
     nmColClasses <- names(colClasses)
     if(length(colClasses) < cols)
         if(is.null(nmColClasses)) {
-            colClasses <- rep(colClasses, length.out=cols)
+            colClasses <- rep_len(colClasses, cols)
         } else {
-            tmp <- rep(NA_character_, length.out=cols)
+            tmp <- rep_len(NA_character_, cols)
             names(tmp) <- col.names
             i <- match(nmColClasses, col.names, 0L)
             if(any(i <= 0L))
@@ -185,7 +185,7 @@ read.DIF <- function(file, header = FALSE, dec = ".",
     }
 
     if(is.logical(as.is)) {
-	as.is <- rep(as.is, length.out=cols)
+	as.is <- rep_len(as.is, cols)
     } else if(is.numeric(as.is)) {
 	if(any(as.is < 1 | as.is > cols))
 	    stop("invalid numeric 'as.is' expression")
