@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2013   The R Core Team.
+ *  Copyright (C) 1998-2014   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,13 +40,17 @@
 #include <errno.h>
 #include <Print.h>
 
-static R_INLINE int imin2(int x, int y)
-{
-    return (x < y) ? x : y;
-}
-
 #include <rlocale.h> /* for btowc */
 
+#undef _
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#define _(String) dgettext ("utils", String)
+#else
+#define _(String) (String)
+#endif
+
+
 /* The size of vector initially allocated by scan */
 #define SCAN_BLOCKSIZE		1000
 /* The size of the console buffer */
@@ -879,6 +883,8 @@ SEXP readtablehead(SEXP args)
 	if(!empty || (c != R_EOF && !blskip)) { /* see previous comment */
 	    SET_STRING_ELT(ans, nread, mkChar(buf));
 	    nread++;
+	    if (strlen(buf) < nbuf) // PR#15625
+		warning("line %d appears to contain embedded nulls", nread);
 	}
 	if(c == R_EOF) goto no_more_lines;
     }
@@ -954,6 +960,7 @@ static const char
     if (indx < 0 || indx >= length(x))
 	error(_("index out of range"));
     if(TYPEOF(x) == STRSXP) {
+	const void *vmax = vmaxget();
 	p0 = translateChar(STRING_ELT(x, indx));
 	if(!quote) return p0;
 	for(nbuf = 2, p = p0; *p; p++) /* find buffer length needed */
@@ -965,6 +972,7 @@ static const char
 	    *q++ = *p++;
 	}
 	*q++ = '"'; *q = '\0';
+	vmaxset(vmax);
 	return buff->data;
     }
     return EncodeElement(x, indx, quote ? '"' : 0, cdec);
@@ -989,10 +997,9 @@ static void wt_cleanup(void *data)
 SEXP writetable(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     SEXP x, sep, rnames, eol, na, dec, quote, xj;
-    int nr, nc, i, j, qmethod;
     Rboolean wasopen, quote_rn = FALSE, *quote_col;
     Rconnection con;
-    const char *csep, *ceol, *cna, *sdec, *tmp=NULL /* -Wall */;
+    const char *csep, *ceol, *cna, *sdec, *tmp = NULL /* -Wall */;
     char cdec;
     SEXP *levels;
     R_StringBuffer strBuf = {NULL, 0, MAXELTSIZE};
@@ -1013,15 +1020,15 @@ SEXP writetable(SEXP call, SEXP op, SEXP args, SEXP env)
 	strcpy(con->mode, "wt");
 	if(!con->open(con)) error(_("cannot open the connection"));
     }
-    nr = asInteger(CAR(args));	   args = CDR(args);
-    nc = asInteger(CAR(args));	   args = CDR(args);
+    int nr = asInteger(CAR(args)); args = CDR(args);
+    int nc = asInteger(CAR(args)); args = CDR(args);
     rnames = CAR(args);		   args = CDR(args);
     sep = CAR(args);		   args = CDR(args);
     eol = CAR(args);		   args = CDR(args);
     na = CAR(args);		   args = CDR(args);
     dec = CAR(args);		   args = CDR(args);
     quote = CAR(args);		   args = CDR(args);
-    qmethod = asLogical(CAR(args));
+    int qmethod = asLogical(CAR(args));
 
     if(nr == NA_INTEGER) error(_("invalid '%s' argument"), "nr");
     if(nc == NA_INTEGER) error(_("invalid '%s' argument"), "nc");
@@ -1040,8 +1047,8 @@ SEXP writetable(SEXP call, SEXP op, SEXP args, SEXP env)
 	error(_("'dec' must be a single character"));
     cdec = sdec[0];
     quote_col = (Rboolean *) R_alloc(nc, sizeof(Rboolean));
-    for(j = 0; j < nc; j++) quote_col[j] = FALSE;
-    for(i = 0; i < length(quote); i++) { /* NB, quote might be NULL */
+    for(int j = 0; j < nc; j++) quote_col[j] = FALSE;
+    for(int i = 0; i < length(quote); i++) { /* NB, quote might be NULL */
 	int this = INTEGER(quote)[i];
 	if(this == 0) quote_rn = TRUE;
 	if(this >  0) quote_col[this - 1] = TRUE;
@@ -1061,7 +1068,7 @@ SEXP writetable(SEXP call, SEXP op, SEXP args, SEXP env)
 
 	/* handle factors internally, check integrity */
 	levels = (SEXP *) R_alloc(nc, sizeof(SEXP));
-	for(j = 0; j < nc; j++) {
+	for(int j = 0; j < nc; j++) {
 	    xj = VECTOR_ELT(x, j);
 	    if(LENGTH(xj) != nr)
 		error(_("corrupt data frame -- length of column %d does not not match nrows"), j+1);
@@ -1070,13 +1077,13 @@ SEXP writetable(SEXP call, SEXP op, SEXP args, SEXP env)
 	    } else levels[j] = R_NilValue;
 	}
 
-	for(i = 0; i < nr; i++) {
+	for(int i = 0; i < nr; i++) {
 	    if(i % 1000 == 999) R_CheckUserInterrupt();
 	    if(!isNull(rnames))
 		Rconn_printf(con, "%s%s",
 			     EncodeElement2(rnames, i, quote_rn, qmethod,
 					    &strBuf, cdec), csep);
-	    for(j = 0; j < nc; j++) {
+	    for(int j = 0; j < nc; j++) {
 		xj = VECTOR_ELT(x, j);
 		if(j > 0) Rconn_printf(con, "%s", csep);
 		if(isna(xj, i)) tmp = cna;
@@ -1111,16 +1118,16 @@ SEXP writetable(SEXP call, SEXP op, SEXP args, SEXP env)
 	if(!isVectorAtomic(x))
 	    UNIMPLEMENTED_TYPE("write.table, matrix method", x);
 	/* quick integrity check */
-	if(LENGTH(x) != nr * nc)
+	if(XLENGTH(x) != (R_len_t)nr * nc)
 	    error(_("corrupt matrix -- dims not not match length"));
 
-	for(i = 0; i < nr; i++) {
+	for(int i = 0; i < nr; i++) {
 	    if(i % 1000 == 999) R_CheckUserInterrupt();
 	    if(!isNull(rnames))
 		Rconn_printf(con, "%s%s",
 			     EncodeElement2(rnames, i, quote_rn, qmethod,
 					    &strBuf, cdec), csep);
-	    for(j = 0; j < nc; j++) {
+	    for(int j = 0; j < nc; j++) {
 		if(j > 0) Rconn_printf(con, "%s", csep);
 		if(isna(x, i + j*nr)) tmp = cna;
 		else {
