@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2012   The R Core Team
+ *  Copyright (C) 1998-2013   The R Core Team
  *  Copyright (C) 2002-2008   The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -111,10 +111,20 @@ SEXP attribute_hidden do_matrix(SEXP call, SEXP op, SEXP args, SEXP rho)
 	nr = (int) lendat;
     } else if (miss_nr) {
 	if (lendat > (double) nc * INT_MAX) error("data is too long");
-	nr = (int) ceil((double) lendat / (double) nc);
+	// avoid division by zero
+	if (nc == 0) {
+	    if (lendat) error(_("nc = 0 for non-null data"));
+	    else nr = 0;
+	} else
+	    nr = (int) ceil((double) lendat / (double) nc);
     } else if (miss_nc) {
 	if (lendat > (double) nr * INT_MAX) error("data is too long");
-	nc = (int) ceil((double) lendat / (double) nr);
+	// avoid division by zero
+	if (nr == 0) {
+	    if (lendat) error(_("nr = 0 for non-null data"));
+	    else nc = 0;
+	} else
+	    nc = (int) ceil((double) lendat / (double) nr);
     }
 
     if(lendat > 0) {
@@ -1267,7 +1277,7 @@ SEXP attribute_hidden do_colsum(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     if (OP == 0 || OP == 1) { /* columns */
 	PROTECT(ans = allocVector(REALSXP, p));
-#ifdef HAVE_OPENMP
+#ifdef _OPENMP
 	int nthreads;
 	/* This gives a spurious -Wunused-but-set-variable error */
 	if (R_num_math_threads > 0)
@@ -1584,7 +1594,6 @@ SEXP attribute_hidden do_backsolve(SEXP call, SEXP op, SEXP args, SEXP rho)
     if (upper == NA_INTEGER) error(_("invalid '%s' argument"), "upper.tri");
     int trans = asLogical(CAR(args));
     if (trans == NA_INTEGER) error(_("invalid '%s' argument"), "transpose");
-    SEXP ans;
     if (TYPEOF(r) != REALSXP) {PROTECT(r = coerceVector(r, REALSXP)); nprot++;}
     if (TYPEOF(b) != REALSXP) {PROTECT(b = coerceVector(b, REALSXP)); nprot++;}
     double *rr = REAL(r);
@@ -1597,7 +1606,7 @@ SEXP attribute_hidden do_backsolve(SEXP call, SEXP op, SEXP args, SEXP rho)
 		  i + 1);
     }
 
-    PROTECT(ans = allocMatrix(REALSXP, k, ncb));
+    SEXP ans = PROTECT(allocMatrix(REALSXP, k, ncb));
     if (k > 0 && ncb > 0) {
        /* copy (part) cols of b to ans */
 	for(R_xlen_t j = 0; j < ncb; j++)
@@ -1617,9 +1626,8 @@ SEXP attribute_hidden do_maxcol(SEXP call, SEXP op, SEXP args, SEXP rho)
     SEXP m = CAR(args);
     int method = asInteger(CADR(args));
     int nr = nrows(m), nc = ncols(m), nprot = 1;
-    if (TYPEOF(m) != REALSXP) PROTECT(m = coerceVector(m, REALSXP));
-    SEXP ans = allocVector(INTSXP, nr);
-    PROTECT(ans);
+    if (TYPEOF(m) != REALSXP) {PROTECT(m = coerceVector(m, REALSXP)); nprot++;}
+    SEXP ans = PROTECT(allocVector(INTSXP, nr));
     R_max_col(REAL(m), &nr, &nc, INTEGER(ans), &method);
     UNPROTECT(nprot);
     return ans;
