@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996, 1997  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 2000-12	    The R Core Team.
+ *  Copyright (C) 2000-2013	    The R Core Team
  *  Copyright (C) 2005		    The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -957,6 +957,7 @@ static void R_cpolyroot(double *opr, double *opi, int *degree,
     if (nn == 1) return;
 
     /* Use a single allocation as these as small */
+    const void *vmax = vmaxget();
     tmp = (double *) R_alloc((size_t) (10*nn), sizeof(double));
     pr = tmp; pi = tmp + nn; hr = tmp + 2*nn; hi = tmp + 3*nn;
     qpr = tmp + 4*nn; qpi = tmp + 5*nn; qhr = tmp + 6*nn; qhi = tmp + 7*nn;
@@ -1022,6 +1023,7 @@ static void R_cpolyroot(double *opr, double *opi, int *degree,
 	/* return empty handed */
 
 	*fail = TRUE;
+	vmaxset(vmax);
 	return;
 
 	/* the second stage jumps directly to the third stage iteration.
@@ -1041,6 +1043,7 @@ static void R_cpolyroot(double *opr, double *opi, int *degree,
     /*	calculate the final zero and return */
     cdivid(-pr[1], -pi[1], pr[0], pi[0], &zeror[d1], &zeroi[d1]);
 
+    vmaxset(vmax);
     return;
 }
 
