@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Langage for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 2001-4  The R Development Core Team
+ *  Copyright (C) 2001-2013  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -59,6 +59,7 @@ SEXP attribute_hidden mkPRIMSXP(int offset, int eval)
     if (result == R_NilValue) {
 	result = allocSExp(type);
 	SET_PRIMOFFSET(result, offset);
+	SET_VECTOR_ELT(PrimCache, offset, result);
     }
     else if (TYPEOF(result) != type)
 	error("requested primitive type is not consistent with cached value");
@@ -86,19 +87,22 @@ SEXP attribute_hidden mkCLOSXP(SEXP formals, SEXP body, SEXP rho)
     if(isList(formals))
 	SET_FORMALS(c, formals);
     else
-	error(_("invalid formal arguments for \"function\""));
+	error(_("invalid formal arguments for 'function'"));
 #else
     SET_FORMALS(c, formals);
 #endif
-    if(isList(body) || isLanguage(body) || isSymbol(body)
-       || isExpression(body) || isVector(body)
-#ifdef BYTECODE
-       || isByteCode(body)
-#endif
-       )
-	SET_BODY(c, body);
-    else
+    switch (TYPEOF(body)) {
+    case CLOSXP:
+    case BUILTINSXP:
+    case SPECIALSXP:
+    case DOTSXP:
+    case ANYSXP:
 	error(_("invalid body argument for 'function'"));
+	break;
+    default:
+	SET_BODY(c, body);
+	break;
+    }
 
     if(rho == R_NilValue)
 	SET_CLOENV(c, R_GlobalEnv);
