@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2011  The R Core Team.
+ *  Copyright (C) 1997--2013  The R Core Team.
  *  Copyright (C) 2003--2011  The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -167,20 +167,35 @@ static void format_via_sprintf(double r, int d, int *kpower, int *nsig)
 }
 
 
+#if defined(HAVE_LONG_DOUBLE) && (SIZEOF_LONG_DOUBLE > SIZEOF_DOUBLE)
 static const long double tbl[] =
 {
-    /* Powers exactly representable with 64 bit mantissa */
+    /* Powers exactly representable with 64 bit mantissa (except the first, which is only used with digits=0) */
+    1e-1, 
     1e00, 1e01, 1e02, 1e03, 1e04, 1e05, 1e06, 1e07, 1e08, 1e09,
     1e10, 1e11, 1e12, 1e13, 1e14, 1e15, 1e16, 1e17, 1e18, 1e19,
     1e20, 1e21, 1e22, 1e23, 1e24, 1e25, 1e26, 1e27
 };
+#define KP_MAX 27
+#else
+static const double tbl[] =
+{
+    1e-1, 
+    1e00, 1e01, 1e02, 1e03, 1e04, 1e05, 1e06, 1e07, 1e08, 1e09,
+    1e10, 1e11, 1e12, 1e13, 1e14, 1e15, 1e16, 1e17, 1e18, 1e19,
+    1e20, 1e21, 1e22
+};
+#define KP_MAX 22
+#endif
 
-static void scientific(double *x, int *sgn, int *kpower, int *nsig)
+static void 
+scientific(double *x, int *sgn, int *kpower, int *nsig, int *roundingwidens)
 {
     /* for a number x , determine
      *	sgn    = 1_{x < 0}  {0/1}
      *	kpower = Exponent of 10;
      *	nsig   = min(R_print.digits, #{significant digits of alpha})
+     *  roundingwidens = 1 if rounding causes x to increase in width, 0 otherwise
      *
      * where  |x| = alpha * 10^kpower	and	 1 <= alpha < 10
      */
@@ -193,6 +208,7 @@ static void scientific(double *x, int *sgn, int *kpower, int *nsig)
 	*kpower = 0;
 	*nsig = 1;
 	*sgn = 0;
+	*roundingwidens = 0;
     } else {
 	if(*x < 0.0) {
 	    *sgn = 1; r = -*x;
@@ -201,6 +217,7 @@ static void scientific(double *x, int *sgn, int *kpower, int *nsig)
 	}
         if (R_print.digits >= DBL_DIG + 1) {
             format_via_sprintf(r, R_print.digits, kpower, nsig);
+	    *roundingwidens = 0;
             return;
         }
         kp = (int) floor(log10(r)) - R_print.digits + 1;/* r = |x|; 10^(kp + digits - 1) <= r */
@@ -208,7 +225,7 @@ static void scientific(double *x, int *sgn, int *kpower, int *nsig)
         long double r_prec = r;
         /* use exact scaling factor in long double precision, if possible */
         if (abs(kp) <= 27) {
-            if (kp > 0) r_prec /= tbl[kp]; else if (kp < 0) r_prec *= tbl[ -kp];
+            if (kp > 0) r_prec /= tbl[kp+1]; else if (kp < 0) r_prec *= tbl[ -kp+1];
         }
 #ifdef HAVE_POWL
 	else
@@ -219,7 +236,7 @@ static void scientific(double *x, int *sgn, int *kpower, int *nsig)
         else
             r_prec /= pow(10.0, (double) kp);
 #endif
-        if (r_prec < tbl[R_print.digits - 1]) {
+        if (r_prec < tbl[R_print.digits]) {
             r_prec *= 10.0;
             kp--;
         }
@@ -228,9 +245,10 @@ static void scientific(double *x, int *sgn, int *kpower, int *nsig)
 	   alpha already rounded to 64 bits */
         alpha = (double) R_nearbyintl(r_prec);
 #else
+	double r_prec = r;
         /* use exact scaling factor in double precision, if possible */
         if (abs(kp) <= 22) {
-            if (kp >= 0) r /= tbl[kp]; else r *= tbl[ -kp];
+            if (kp >= 0) r_prec /= tbl[kp+1]; else r_prec *= tbl[ -kp+1];
         }
         /* on IEEE 1e-308 is not representable except by gradual underflow.
            Shifting by 303 allows for any potential denormalized numbers x,
@@ -238,16 +256,17 @@ static void scientific(double *x, int *sgn, int *kpower, int *nsig)
            is in range. Representation of 1e+303 has low error.
          */
         else if (kp <= R_dec_min_exponent)
-            r = (r * 1e+303)/pow(10.0, (double)(kp+303));
+            r_prec = (r_prec * 1e+303)/pow(10.0, (double)(kp+303));
         else
-            r /= pow(10.0, (double)kp);
-        if (r < tbl[R_print.digits - 1]) {
-            r *= 10.0;
+            r_prec /= pow(10.0, (double)kp);
+        if (r_prec < tbl[R_print.digits]) {
+            r_prec *= 10.0;
             kp--;
         }
         /* round alpha to integer, 10^(digits-1) <= alpha <= 10^digits */
-        /* accuracy limited by double rounding problem, alpha already rounded to 53 bits */
-        alpha = R_nearbyint(r);
+        /* accuracy limited by double rounding problem, 
+	   alpha already rounded to 53 bits */
+        alpha = R_nearbyint(r_prec);
 #endif
         *nsig = R_print.digits;
         for (j = 1; j <= R_print.digits; j++) {
@@ -258,11 +277,24 @@ static void scientific(double *x, int *sgn, int *kpower, int *nsig)
                 break;
             }
         }
-        if (*nsig == 0) {
+        if (*nsig == 0 && R_print.digits > 0) {
             *nsig = 1;
             kp += 1;
         }
         *kpower = kp + R_print.digits - 1;
+
+	/* Scientific format may do more rounding than fixed format, e.g.
+	   9996 with 3 digits is 1e+04 in scientific, but 9996 in fixed.
+	   This happens when the true value r is less than 10^(kpower+1)
+	   and would not round up to it in fixed format.
+	   Here rgt is the decimal place that will be cut off by rounding */
+	   
+	int rgt = R_print.digits - *kpower;
+	/* bound rgt by 0 and KP_MAX */
+	rgt = rgt < 0 ? 0 : rgt > KP_MAX ? KP_MAX : rgt;
+	double fuzz = 0.5/(double)tbl[1 + rgt];
+	// kpower can be bigger than the table.	
+	*roundingwidens = *kpower > 0 && *kpower <= KP_MAX && r < tbl[*kpower + 1] - fuzz;
     }
 }
 
@@ -280,7 +312,7 @@ void formatReal(double *x, R_xlen_t n, int *w, int *d, int *e, int nsmall)
 {
     int left, right, sleft;
     int mnl, mxl, rgt, mxsl, mxns, wF;
-    int neg, sgn, kpower, nsig;
+    int neg, sgn, kpower, nsig, roundingwidens;
     int naflag, nanflag, posinf, neginf;
 
     nanflag = 0;
@@ -298,9 +330,11 @@ void formatReal(double *x, R_xlen_t n, int *w, int *d, int *e, int nsmall)
 	    else if(x[i] > 0) posinf = 1;
 	    else neginf = 1;
 	} else {
-	    scientific(&x[i], &sgn, &kpower, &nsig);
+	    scientific(&x[i], &sgn, &kpower, &nsig, &roundingwidens);
 
 	    left = kpower + 1;
+	    if (roundingwidens) left--;
+	    
 	    sleft = sgn + ((left <= 0) ? 1 : left); /* >= 1 */
 	    right = nsig - left; /* #{digits} right of '.' ( > 0 often)*/
 	    if (sgn) neg = 1;	 /* if any < 0, need extra space for sign */
@@ -324,6 +358,7 @@ void formatReal(double *x, R_xlen_t n, int *w, int *d, int *e, int nsmall)
 
     /*-- These	'mxsl' & 'rgt'	are used in F Format
      *	 AND in the	____ if(.) "F" else "E" ___   below: */
+    if (R_print.digits == 0) rgt = 0;
     if (mxl < 0) mxsl = 1 + neg;  /* we use %#w.dg, so have leading zero */
 
     /* use nsmall only *after* comparing "F" vs "E": */
@@ -332,7 +367,7 @@ void formatReal(double *x, R_xlen_t n, int *w, int *d, int *e, int nsmall)
 
     /*-- 'see' how "E" Exponential format would be like : */
     *e = (mxl > 100 || mnl <= -99) ? 2 /* 3 digit exponent */ : 1;
-    if (mxns > 0) {
+    if (mxns != INT_MIN) {
 	*d = mxns - 1;
 	*w = neg + (*d > 0) + *d + 4 + *e; /* width for E format */
 	if (wF <= *w + R_print.scipen) { /* Fixpoint if it needs less space */
@@ -356,7 +391,6 @@ void formatReal(double *x, R_xlen_t n, int *w, int *d, int *e, int nsmall)
     if (posinf && *w < 3) *w = 3;
     if (neginf && *w < 4) *w = 4;
 }
-
 
 /* As from 2.2.0 the number of digits applies to real and imaginary parts
    together, not separately */
@@ -370,7 +404,7 @@ void formatComplex(Rcomplex *x, R_xlen_t n, int *wr, int *dr, int *er,
     int rt, mnl, mxl, mxsl, mxns, wF, i_wF;
     int i_rt, i_mnl, i_mxl, i_mxsl, i_mxns;
     int neg, sgn;
-    int kpower, nsig;
+    int kpower, nsig, roundingwidens;
     int naflag;
     int rnanflag, rposinf, rneginf, inanflag, iposinf;
     Rcomplex tmp;
@@ -402,9 +436,10 @@ void formatComplex(Rcomplex *x, R_xlen_t n, int *wr, int *dr, int *er,
 		else rneginf = 1;
 	    } else {
 		if(x[i].r != 0) all_re_zero = FALSE;
-		scientific(&(tmp.r), &sgn, &kpower, &nsig);
+		scientific(&(tmp.r), &sgn, &kpower, &nsig, &roundingwidens);
 
 		left = kpower + 1;
+		if (roundingwidens) left--;
 		sleft = sgn + ((left <= 0) ? 1 : left); /* >= 1 */
 		right = nsig - left; /* #{digits} right of '.' ( > 0 often)*/
 		if (sgn) neg = 1; /* if any < 0, need extra space for sign */
@@ -426,9 +461,10 @@ void formatComplex(Rcomplex *x, R_xlen_t n, int *wr, int *dr, int *er,
 		else iposinf = 1;
 	    } else {
 		if(x[i].i != 0) all_im_zero = FALSE;
-		scientific(&(tmp.i), &sgn, &kpower, &nsig);
+		scientific(&(tmp.i), &sgn, &kpower, &nsig, &roundingwidens);
 
 		left = kpower + 1;
+		if (roundingwidens) left--;
 		sleft = ((left <= 0) ? 1 : left);
 		right = nsig - left;
 
@@ -446,6 +482,7 @@ void formatComplex(Rcomplex *x, R_xlen_t n, int *wr, int *dr, int *er,
 
     /* overall format for real part	*/
 
+    if (R_print.digits == 0) rt = 0;
     if (mxl != INT_MIN) {
 	if (mxl < 0) mxsl = 1 + neg;
 	if (rt < 0) rt = 0;
@@ -463,6 +500,7 @@ void formatComplex(Rcomplex *x, R_xlen_t n, int *wr, int *dr, int *er,
 
     /* overall format for imaginary part */
 
+    if (R_print.digits == 0) i_rt = 0;
     if (i_mxl != INT_MIN) {
 	if (i_mxl < 0) i_mxsl = 1;
 	if (i_rt < 0) i_rt = 0;
