@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996	Robert Gentleman and Ross Ihaka
- *  Copyright (C) 2000--2012	The R Core Team
+ *  Copyright (C) 2000--2013	The R Core Team
  *  Copyright (C) 2001--2012	The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -165,9 +165,11 @@ static void printLogicalMatrix(SEXP sx, int offset, int r_pr, int r, int c,
 #	define _PRINT_SET_clabw					\
 								\
 	if (!isNull(cl)) {					\
+	    const void *vmax = vmaxget();			\
 	    if(STRING_ELT(cl, j) == NA_STRING)			\
 		clabw = R_print.na_width_noquote;		\
 	    else clabw = strwidth(translateChar(STRING_ELT(cl, j)));	\
+	    vmaxset(vmax);					\
 	} else							\
 	    clabw = IndexWidth(j + 1) + 3;
 
@@ -429,7 +431,7 @@ static void printRawMatrix(SEXP sx, int offset, int r_pr, int r, int c,
 	for (i = 0; i < r_pr; i++) {
 	    MatrixRowLabel(rl, i, rlabw, lbloff);
 	    for (j = jmin; j < jmax; j++)
-		Rprintf("%*s%s", w[j]-2, "", EncodeRaw(x[i + j * r]));
+		Rprintf("%*s%s", w[j]-2, "", EncodeRaw(x[i + j * r], ""));
 	}
 	Rprintf("\n");
 	jmin = jmax;
@@ -443,6 +445,7 @@ void printMatrix(SEXP x, int offset, SEXP dim, int quote, int right,
 /* 'rl' and 'cl' are dimnames(.)[[1]] and dimnames(.)[[2]]  whereas
  * 'rn' and 'cn' are the  names(dimnames(.))
  */
+    const void *vmax = vmaxget();
     int r = INTEGER(dim)[0];
     int c = INTEGER(dim)[1], r_pr;
     /* PR#850 */
@@ -492,13 +495,14 @@ void printMatrix(SEXP x, int offset, SEXP dim, int quote, int right,
 	Rprintf(" [ reached getOption(\"max.print\") -- omitted %d rows ]\n",
 		r - r_pr);
 #endif
+    vmaxset(vmax);
 }
 
 attribute_hidden
 void printArray(SEXP x, SEXP dim, int quote, int right, SEXP dimnames)
 {
 /* == printArray(.) */
-
+    const void *vmax = vmaxget();
     int ndim = LENGTH(dim);
     const char *rn = NULL, *cn = NULL;
 
@@ -604,5 +608,6 @@ void printArray(SEXP x, SEXP dim, int quote, int right, SEXP dimnames)
 	    Rprintf(" %d matrix slice(s) ]\n", nb - nb_pr);
 	}
     }
+    vmaxset(vmax);
 }
 
