@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2012   The R Core Team.
+ *  Copyright (C) 1998-2013   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -41,11 +41,6 @@
 #include <errno.h>
 #include <Print.h>
 
-static R_INLINE int imin2(int x, int y)
-{
-    return (x < y) ? x : y;
-}
-
 #include <rlocale.h> /* for btowc */
 
 /* The size of vector initially allocated by scan */
@@ -80,6 +75,7 @@ typedef struct {
     Rboolean isLatin1; /* = FALSE */
     Rboolean isUTF8; /* = FALSE */
     Rboolean atStart;
+    Rboolean embedWarn;
     char convbuf[100];
 } LocalData;
 
@@ -214,8 +210,10 @@ strtoraw (const char *nptr, char **endptr)
 
 static R_INLINE int scanchar_raw(LocalData *d)
 {
-    return (d->ttyflag) ? ConsoleGetcharWithPushBack(d->con) :
+    int c = (d->ttyflag) ? ConsoleGetcharWithPushBack(d->con) :
 	Rconn_fgetc(d->con);
+    if(c == 0) d->embedWarn = TRUE;
+    return c;
 }
 
 static R_INLINE void unscanchar(int c, LocalData *d)
@@ -354,6 +352,8 @@ fillBuffer(SEXPTYPE type, int strip, int *bch, LocalData *d,
 		if(dbcslocale && btowc(c) == WEOF)
 		    buffer->data[m++] = (char) scanchar2(d);
 	    }
+	    if (c == R_EOF)
+	    	warning(_("EOF within quoted string"));
 	    c = scanchar(FALSE, d);
 	    mm = m;
 	}
@@ -402,6 +402,8 @@ fillBuffer(SEXPTYPE type, int strip, int *bch, LocalData *d,
 			if(dbcslocale && btowc(c) == WEOF)
 			    buffer->data[m++] = (char) scanchar2(d);
 		    }
+		    if (c == R_EOF)
+		    	warning(_("EOF within quoted string"));
 		    c = scanchar(TRUE, d); /* only peek at lead byte
 					      unless ASCII */
 		    if (c == quote) {
@@ -816,7 +818,7 @@ SEXP attribute_hidden do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
     const char *p, *encoding;
     RCNTXT cntxt;
     LocalData data = {NULL, 0, 0, '.', NULL, NO_COMCHAR, 0, NULL, FALSE,
-		      FALSE, 0, FALSE, FALSE};
+		      FALSE, 0, FALSE, FALSE, FALSE};
     data.NAstrings = R_NilValue;
 
     checkArity(op, args);
@@ -968,6 +970,8 @@ SEXP attribute_hidden do_scan(SEXP call, SEXP op, SEXP args, SEXP rho)
     if (!data.ttyflag && !data.wasopen)
 	data.con->close(data.con);
     if (data.quoteset[0]) free(data.quoteset);
+    if (data.embedWarn) 
+	warning(_("embedded nul(s) found in input"));
     return ans;
 }
 
