@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Langage for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 2001-13     The R Core Team
+ *  Copyright (C) 2001-2013   The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -144,7 +144,7 @@ void parseError(SEXP call, int linenum)
 	    error("%s%d:%d: %s\n%d: %s\n%*s",
 		  filename, linenum, R_ParseErrorCol, R_ParseErrorMsg,
 		  R_ParseContextLine, CHAR(STRING_ELT(context, 0)), 
-		  width+R_ParseErrorCol, "^");
+		  width+R_ParseErrorCol+1, "^");
 	    break;
 	default:
 	    width = snprintf(buffer, 10, "%d:", R_ParseContextLine);
@@ -152,7 +152,7 @@ void parseError(SEXP call, int linenum)
 		  filename, linenum, R_ParseErrorCol, R_ParseErrorMsg,
 		  R_ParseContextLine-1, CHAR(STRING_ELT(context, len-2)),
 		  R_ParseContextLine, CHAR(STRING_ELT(context, len-1)), 
-		  width+R_ParseErrorCol, "^");
+		  width+R_ParseErrorCol+1, "^");
 	    break;
 	}
     } else {
