@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2012  The R Core Team
+ *  Copyright (C) 1997--2013  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *
  *  OneIndex()        -- used for "[[<-" in ./subassign.c
  *  get1index()       -- used for "[["   in ./subassign.c & subset.c
- *  vectorIndex()     -- used for "[[" with a vector arg
+ *  vectorIndex()     -- used for "[[" and "[[<-" with a vector arg
 
  *  mat2indsub()      -- for "mat[i]"     "    "            "
 
@@ -67,6 +67,7 @@ OneIndex(SEXP x, SEXP s, R_xlen_t len, int partial, SEXP *newname,
 {
     SEXP names;
     R_xlen_t i, indx, nx;
+    const void *vmax;
 
     if (pos < 0 && length(s) > 1) {
 	ECALL(call, _("attempt to select more than one element"));
@@ -88,6 +89,7 @@ OneIndex(SEXP x, SEXP s, R_xlen_t len, int partial, SEXP *newname,
 	indx = integerOneIndex((int)REAL(s)[pos], len, call);
 	break;
     case STRSXP:
+	vmax = vmaxget();
 	nx = xlength(x);
 	names = getAttrib(x, R_NamesSymbol);
 	if (names != R_NilValue) {
@@ -118,8 +120,10 @@ OneIndex(SEXP x, SEXP s, R_xlen_t len, int partial, SEXP *newname,
 	if (indx == -1)
 	    indx = nx;
 	*newname = STRING_ELT(s, pos);
+	vmaxset(vmax);
 	break;
     case SYMSXP:
+	vmax = vmaxget();
 	nx = xlength(x);
 	names = getAttrib(x, R_NamesSymbol);
 	if (names != R_NilValue) {
@@ -133,6 +137,7 @@ OneIndex(SEXP x, SEXP s, R_xlen_t len, int partial, SEXP *newname,
 	if (indx == -1)
 	    indx = nx;
 	*newname = STRING_ELT(s, pos);
+	vmaxset(vmax);
 	break;
     default:
 	if (call == R_NilValue)
@@ -162,6 +167,7 @@ get1index(SEXP s, SEXP names, R_xlen_t len, int pok, int pos, SEXP call)
     int  warn_pok = 0;
     const char *ss, *cur_name;
     R_xlen_t indx;
+    const void *vmax;
 
     if (pok == -1) {
 	pok = 1;
@@ -212,6 +218,7 @@ get1index(SEXP s, SEXP names, R_xlen_t len, int pok, int pos, SEXP call)
 	if(!CHAR(STRING_ELT(s, pos))[0]) break;
 
 	/* Try for exact match */
+	vmax = vmaxget();
 	ss = translateChar(STRING_ELT(s, pos));
 	for (R_xlen_t i = 0; i < xlength(names); i++)
 	    if (STRING_ELT(names, i) != NA_STRING) {
@@ -251,13 +258,16 @@ get1index(SEXP s, SEXP names, R_xlen_t len, int pok, int pos, SEXP call)
 		}
 	    }
 	}
+	vmaxset(vmax);
 	break;
     case SYMSXP:
+	vmax = vmaxget();
 	for (R_xlen_t i = 0; i < xlength(names); i++)
 	    if (STRING_ELT(names, i) != NA_STRING &&
 		streql(translateChar(STRING_ELT(names, i)),
 		       CHAR(PRINTNAME(s)))) {
 		indx = i;
+		vmaxset(vmax);
 		break;
 	    }
     default:
@@ -273,12 +283,19 @@ get1index(SEXP s, SEXP names, R_xlen_t len, int pok, int pos, SEXP call)
 /* This is used for [[ and [[<- with a vector of indices of length > 1 .
    x is a list or pairlist, and it is indexed recusively from 
    level start to level stop-1.  ( 0...len-1 or 0..len-2 then len-1).
+   For [[<- it needs to duplicate if substructure has NAMED > 1.
  */
 SEXP attribute_hidden
-vectorIndex(SEXP x, SEXP thesub, int start, int stop, int pok, SEXP call) 
+vectorIndex(SEXP x, SEXP thesub, int start, int stop, int pok, SEXP call,
+	    Rboolean dup) 
 {
     int i;
     R_xlen_t offset;
+    SEXP cx;
+
+    /* sanity check */
+    if (dup && NAMED(x) > 1)
+	error("should only be called in an assignment context.");
 
     for(i = start; i < stop; i++) {
 	if(!isVectorList(x) && !isPairList(x)) {
@@ -296,9 +313,19 @@ vectorIndex(SEXP x, SEXP thesub, int start, int stop, int pok, SEXP call)
 	    if (offset > R_SHORT_LEN_MAX)
 		error("invalid subscript for pairlist");
 #endif
-	    x = CAR(nthcdr(x, (int) offset));
+	    cx = nthcdr(x, (int) offset);
+	    x = CAR(cx);
+	    if (dup && NAMED(x) > 1) {
+		x = duplicate(x);
+		SETCAR(cx, x);
+	    }
 	} else {
+	    cx = x;
 	    x = VECTOR_ELT(x, offset);
+	    if (dup && NAMED(x) > 1) {
+		x = duplicate(x);
+		SET_VECTOR_ELT(cx, offset, x);
+	    }
     	}
     }
     return x;
