@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997-2012   The R Core Team
+ *  Copyright (C) 1997-2013   The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -42,22 +42,34 @@
 #define DbgP3(s,a,b)
 #endif
 
-
-/* FIXME: use 64-bit integer accumulator? */
+#ifdef LONG_INT
 static Rboolean isum(int *x, R_xlen_t n, int *value, Rboolean narm, SEXP call)
 {
-    double s = 0.0;
-    R_xlen_t i;
+    LONG_INT s = 0;  // at least 64-bit
     Rboolean updated = FALSE;
+#ifdef LONG_VECTOR_SUPPORT
+    int ii = R_INT_MIN; // need > 2^32 entries to overflow.
+#endif
 
-    for (i = 0; i < n; i++) {
+    for (R_xlen_t i = 0; i < n; i++) {
 	if (x[i] != NA_INTEGER) {
 	    if(!updated) updated = TRUE;
 	    s += x[i];
+#ifdef LONG_VECTOR_SUPPORT
+	    if (ii++ > 1000) {
+		ii = 0;
+		if (s > 9000000000000000L || s < -9000000000000000L) {
+		    if(!updated) updated = TRUE;
+		    *value = NA_INTEGER;
+		    warningcall(call, _("integer overflow - use sum(as.numeric(.))"));
+		    return updated;
+		}
+	    }
+#endif
 	} else if (!narm) {
 	    if(!updated) updated = TRUE;
 	    *value = NA_INTEGER;
-	    return(updated);
+	    return updated;
 	}
     }
     if(s > INT_MAX || s < R_INT_MIN){
@@ -66,16 +78,41 @@ static Rboolean isum(int *x, R_xlen_t n, int *value, Rboolean narm, SEXP call)
     }
     else *value = (int) s;
 
-    return(updated);
+    return updated;
 }
+#else
+/* Version from R 3.0.0: should never be used with a C99/C11 compiler */
+static Rboolean isum(int *x, R_xlen_t n, int *value, Rboolean narm, SEXP call)
+{
+    double s = 0.0;
+    Rboolean updated = FALSE;
+
+    for (R_xlen_t i = 0; i < n; i++) {
+	if (x[i] != NA_INTEGER) {
+	    if(!updated) updated = TRUE;
+	    s += x[i];
+	} else if (!narm) {
+	    if(!updated) updated = TRUE;
+	    *value = NA_INTEGER;
+	    return updated;
+	}
+    }
+    if(s > INT_MAX || s < R_INT_MIN){
+	warningcall(call, _("integer overflow - use sum(as.numeric(.))"));
+	*value = NA_INTEGER;
+    }
+    else *value = (int) s;
+
+    return updated;
+}
+#endif
 
 static Rboolean rsum(double *x, R_xlen_t n, double *value, Rboolean narm)
 {
     LDOUBLE s = 0.0;
-    R_xlen_t i;
     Rboolean updated = FALSE;
 
-    for (i = 0; i < n; i++) {
+    for (R_xlen_t i = 0; i < n; i++) {
 	if (!narm || !ISNAN(x[i])) {
 	    if(!updated) updated = TRUE;
 	    s += x[i];
@@ -83,16 +120,15 @@ static Rboolean rsum(double *x, R_xlen_t n, double *value, Rboolean narm)
     }
     *value = (double) s;
 
-    return(updated);
+    return updated;
 }
 
 static Rboolean csum(Rcomplex *x, R_xlen_t n, Rcomplex *value, Rboolean narm)
 {
     LDOUBLE sr = 0.0, si = 0.0;
-    R_xlen_t i;
     Rboolean updated = FALSE;
 
-    for (i = 0; i < n; i++) {
+    for (R_xlen_t i = 0; i < n; i++) {
 	if (!narm || (!ISNAN(x[i].r) && !ISNAN(x[i].i))) {
 	    if(!updated) updated = TRUE;
 	    sr += x[i].r;
@@ -102,17 +138,16 @@ static Rboolean csum(Rcomplex *x, R_xlen_t n, Rcomplex *value, Rboolean narm)
     value->r = (double) sr;
     value->i = (double) si;
 
-    return(updated);
+    return updated;
 }
 
 static Rboolean imin(int *x, R_xlen_t n, int *value, Rboolean narm)
 {
     int s = 0 /* -Wall */;
-    R_xlen_t i;
     Rboolean updated = FALSE;
 
     /* Used to set s = INT_MAX, but this ignored INT_MAX in the input */
-    for (i = 0; i < n; i++) {
+    for (R_xlen_t i = 0; i < n; i++) {
 	if (x[i] != NA_INTEGER) {
 	    if (!updated || s > x[i]) {
 		s = x[i];
@@ -126,17 +161,16 @@ static Rboolean imin(int *x, R_xlen_t n, int *value, Rboolean narm)
     }
     *value = s;
 
-    return(updated);
+    return updated;
 }
 
 static Rboolean rmin(double *x, R_xlen_t n, double *value, Rboolean narm)
 {
     double s = 0.0; /* -Wall */
-    R_xlen_t i;
     Rboolean updated = FALSE;
 
     /* s = R_PosInf; */
-    for (i = 0; i < n; i++) {
+    for (R_xlen_t i = 0; i < n; i++) {
 	if (ISNAN(x[i])) {/* Na(N) */
 	    if (!narm) {
 		if(!ISNA(s)) s = x[i]; /* so any NA trumps all NaNs */
@@ -150,16 +184,16 @@ static Rboolean rmin(double *x, R_xlen_t n, double *value, Rboolean narm)
     }
     *value = s;
 
-    return(updated);
+    return updated;
 }
 
 static Rboolean smin(SEXP x, SEXP *value, Rboolean narm)
 {
-    R_xlen_t i;
     SEXP s = NA_STRING; /* -Wall */
     Rboolean updated = FALSE;
+    const void *vmax = vmaxget(); // precautionary for Scollate
 
-    for (i = 0; i < XLENGTH(x); i++) {
+    for (R_xlen_t i = 0; i < XLENGTH(x); i++) {
 	if (STRING_ELT(x, i) != NA_STRING) {
 	    if (!updated ||
 		(s != STRING_ELT(x, i) && Scollate(s, STRING_ELT(x, i)) > 0)) {
@@ -174,16 +208,16 @@ static Rboolean smin(SEXP x, SEXP *value, Rboolean narm)
     }
     *value = s;
 
-    return(updated);
+    vmaxset(vmax);
+    return updated;
 }
 
 static Rboolean imax(int *x, R_xlen_t n, int *value, Rboolean narm)
 {
     int s = 0 /* -Wall */;
-    R_xlen_t i;
     Rboolean updated = FALSE;
 
-    for (i = 0; i < n; i++) {
+    for (R_xlen_t i = 0; i < n; i++) {
 	if (x[i] != NA_INTEGER) {
 	    if (!updated || s < x[i]) {
 		s = x[i];
@@ -196,16 +230,15 @@ static Rboolean imax(int *x, R_xlen_t n, int *value, Rboolean narm)
     }
     *value = s;
 
-    return(updated);
+    return updated;
 }
 
 static Rboolean rmax(double *x, R_xlen_t n, double *value, Rboolean narm)
 {
     double s = 0.0 /* -Wall */;
-    R_xlen_t i;
     Rboolean updated = FALSE;
 
-    for (i = 0; i < n; i++) {
+    for (R_xlen_t i = 0; i < n; i++) {
 	if (ISNAN(x[i])) {/* Na(N) */
 	    if (!narm) {
 		if(!ISNA(s)) s = x[i]; /* so any NA trumps all NaNs */
@@ -219,16 +252,16 @@ static Rboolean rmax(double *x, R_xlen_t n, double *value, Rboolean narm)
     }
     *value = s;
 
-    return(updated);
+    return updated;
 }
 
 static Rboolean smax(SEXP x, SEXP *value, Rboolean narm)
 {
-    R_xlen_t i;
     SEXP s = NA_STRING; /* -Wall */
     Rboolean updated = FALSE;
+    const void *vmax = vmaxget(); // precautionary for Scollate
 
-    for (i = 0; i < XLENGTH(x); i++) {
+    for (R_xlen_t i = 0; i < XLENGTH(x); i++) {
 	if (STRING_ELT(x, i) != NA_STRING) {
 	    if (!updated ||
 		(s != STRING_ELT(x, i) && Scollate(s, STRING_ELT(x, i)) < 0)) {
@@ -243,16 +276,16 @@ static Rboolean smax(SEXP x, SEXP *value, Rboolean narm)
     }
     *value = s;
 
-    return(updated);
+    vmaxset(vmax);
+    return updated;
 }
 
 static Rboolean iprod(int *x, R_xlen_t n, double *value, Rboolean narm)
 {
-    double s = 1.0;
-    R_xlen_t i;
+    LDOUBLE s = 1.0;
     Rboolean updated = FALSE;
 
-    for (i = 0; i < n; i++) {
+    for (R_xlen_t i = 0; i < n; i++) {
 	if (x[i] != NA_INTEGER) {
 	    s *= x[i];
 	    if(!updated) updated = TRUE;
@@ -260,26 +293,25 @@ static Rboolean iprod(int *x, R_xlen_t n, double *value, Rboolean narm)
 	else if (!narm) {
 	    if(!updated) updated = TRUE;
 	    *value = NA_REAL;
-	    return(updated);
+	    return updated;
 	}
 
 	if(ISNAN(s)) {  /* how can this happen? */
 	    *value = NA_REAL;
-	    return(updated);
+	    return updated;
 	}
     }
-    *value = s;
+    *value = (double) s;
 
-    return(updated);
+    return updated;
 }
 
 static Rboolean rprod(double *x, R_xlen_t n, double *value, Rboolean narm)
 {
     LDOUBLE s = 1.0;
-    R_xlen_t i;
     Rboolean updated = FALSE;
 
-    for (i = 0; i < n; i++) {
+    for (R_xlen_t i = 0; i < n; i++) {
 	if (!narm || !ISNAN(x[i])) {
 	    if(!updated) updated = TRUE;
 	    s *= x[i];
@@ -287,21 +319,17 @@ static Rboolean rprod(double *x, R_xlen_t n, double *value, Rboolean narm)
     }
     *value = (double) s;
 
-    return(updated);
+    return updated;
 }
 
 static Rboolean cprod(Rcomplex *x, R_xlen_t n, Rcomplex *value, Rboolean narm)
 {
-    LDOUBLE sr, si, tr, ti;
-    R_xlen_t i;
+    LDOUBLE sr = 1.0, si = 0.0;
     Rboolean updated = FALSE;
-    sr = 1;
-    si = 0;
-    for (i = 0; i < n; i++) {
+    for (R_xlen_t i = 0; i < n; i++) {
 	if (!narm || (!ISNAN(x[i].r) && !ISNAN(x[i].i))) {
 	    if(!updated) updated = TRUE;
-	    tr = sr;
-	    ti = si;
+	    LDOUBLE tr = sr, ti = si;
 	    sr = tr * x[i].r - ti * x[i].i;
 	    si = tr * x[i].i + ti * x[i].r;
 	}
@@ -309,7 +337,7 @@ static Rboolean cprod(Rcomplex *x, R_xlen_t n, Rcomplex *value, Rboolean narm)
     value->r = (double) sr;
     value->i = (double) si;
 
-    return(updated);
+    return updated;
 }
 
 
@@ -359,7 +387,7 @@ SEXP attribute_hidden do_summary(SEXP call, SEXP op, SEXP args, SEXP env)
     SEXP ans, a, stmp = NA_STRING /* -Wall */, scum = NA_STRING, call2;
     double tmp = 0.0, s;
     Rcomplex z, ztmp, zcum={0.0, 0.0} /* -Wall */;
-    int itmp = 0, icum=0, int_a, real_a, empty, warn = 0 /* dummy */;
+    int itmp = 0, icum = 0, int_a, real_a, empty, warn = 0 /* dummy */;
     SEXPTYPE ans_type;/* only INTEGER, REAL, COMPLEX or STRSXP here */
 
     Rboolean narm;
@@ -533,8 +561,9 @@ SEXP attribute_hidden do_summary(SEXP call, SEXP op, SEXP args, SEXP env)
 		    DbgP1(" updated:");
 		    if(ans_type == INTSXP) {
 			DbgP3(" INT: (old)icum= %ld, itmp=%ld\n", icum,itmp);
-			if (itmp == NA_INTEGER) goto na_answer;
-			if ((iop == 2 && itmp < icum) || /* min */
+			if (icum == NA_INTEGER); /* NA trumps anything */
+			else if (itmp == NA_INTEGER ||
+			    (iop == 2 && itmp < icum) || /* min */
 			    (iop == 3 && itmp > icum))   /* max */
 			    icum = itmp;
 		    } else if(ans_type == REALSXP) {
@@ -549,12 +578,13 @@ SEXP attribute_hidden do_summary(SEXP call, SEXP op, SEXP args, SEXP env)
 			    (iop == 3 && tmp > zcum.r))	zcum.r = tmp;
 		    } else if(ans_type == STRSXP) {
 			if(empty) scum = stmp;
-			else {
+			else if (scum != NA_STRING) {
 			    if(int_a)
 				stmp = StringFromInteger(itmp, &warn);
 			    if(real_a)
 				stmp = StringFromReal(tmp, &warn);
-			    if(((iop == 2 && stmp != scum && Scollate(stmp, scum) < 0)) ||
+			    if(stmp == NA_STRING ||
+			       (iop == 2 && stmp != scum && Scollate(stmp, scum) < 0) ||
 			       (iop == 3 && stmp != scum && Scollate(stmp, scum) > 0) )
 				scum = stmp;
 			}
@@ -648,7 +678,7 @@ SEXP attribute_hidden do_summary(SEXP call, SEXP op, SEXP args, SEXP env)
 
 		break;/* prod() part */
 
-	    }/* switch(iop) */
+	    } /* switch(iop) */
 
 	} else { /* len(a)=0 */
 	    /* Even though this has length zero it can still be invalid,
@@ -709,7 +739,7 @@ SEXP attribute_hidden do_summary(SEXP call, SEXP op, SEXP args, SEXP env)
     UNPROTECT(1);  /* args */
     return ans;
 
-na_answer: /* only INTSXP case currently used */
+na_answer: /* only sum(INTSXP, ...) case currently used */
     ans = allocVector(ans_type, 1);
     switch(ans_type) {
     case INTSXP:	INTEGER(ans)[0] = NA_INTEGER; break;
