@@ -1,3 +1,21 @@
+/*
+ *  R : A Computer Language for Statistical Data Analysis
+ *  Copyright (C) 2000--2007  R Core Team
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
+ */
 	/* Utilities for `dpq' handling (density/probability/quantile) */
 
 /* give_log in "d";  log_p in "p" & "q" : */
@@ -19,6 +37,9 @@
 #define R_D_log(p)	(log_p	?  (p)	 : log(p))	/* log(p) */
 #define R_D_Clog(p)	(log_p	? log1p(-(p)) : (0.5 - (p) + 0.5)) /* [log](1-p) */
 
+/* log(1 - exp(x))  in more stable form than log1p(- R_D_qIv(x))) : */
+#define R_Log1_Exp(x)   ((x) > -M_LN2 ? log(-expm1(x)) : log1p(-exp(x)))
+
 /* log(1-exp(x)):  R_D_LExp(x) == (log1p(- R_D_qIv(x))) but even more stable:*/
 #define R_D_LExp(x)     (log_p ? R_Log1_Exp(x) : log1p(-x))
 
@@ -91,8 +112,13 @@
 
 /* additions for density functions (C.Loader) */
 #define R_D_fexp(f,x)     (give_log ? -0.5*log(f)+(x) : exp(x)/sqrt(f))
-#define R_D_forceint(x)   floor((x) + 0.5)
-#define R_D_nonint(x) 	  (fabs((x) - floor((x)+0.5)) > 1e-7)
+#ifdef HAVE_NEARYINT
+# define R_D_forceint(x)   nearbyint()
+#define R_D_nonint(x) 	  (fabs((x) - nearbyint(x)) > 1e-7)
+#else
+# define R_D_forceint(x)   round(x)
+#define R_D_nonint(x) 	  (fabs((x) - round(x)) > 1e-7)
+#endif
 /* [neg]ative or [non int]eger : */
 #define R_D_negInonint(x) (x < 0. || R_D_nonint(x))
 
