@@ -11,7 +11,7 @@
  *    distribution function. Appl.Statist., 41, 478-482.
 
  *  Other parts
- *  Copyright (C) 2000-2012  The R Core Team
+ *  Copyright (C) 2000-2013  The R Core Team
  *  Copyright (C) 2003-2009  The R Foundation
  */
 
@@ -26,6 +26,13 @@
  * bad precision & non-convergence in some cases (x ~= f, both LARGE)
  */
 
+#ifdef HAVE_LONG_DOUBLE
+# define EXP expl
+# define FABS fabsl
+#else
+# define EXP exp
+# define FABS fabs
+#endif
 
 double pnchisq(double x, double df, double ncp, int lower_tail, int log_p)
 {
@@ -64,7 +71,7 @@ pnchisq_raw(double x, double f, double theta,
     double l_lam = -1., l_x = -1.; /* initialized for -Wall */
     int n;
     Rboolean lamSml, tSml, is_r, is_b, is_it;
-    long double ans, u, v, t, lt, lu =-1;
+    LDOUBLE ans, u, v, t, lt, lu =-1;
 
     static const double _dbl_min_exp = M_LN2 * DBL_MIN_EXP;
     /*= -708.3964 for IEEE double precision */
@@ -83,7 +90,8 @@ pnchisq_raw(double x, double f, double theta,
 #endif
 
     if(theta < 80) { /* use 110 for Inf, as ppois(110, 80/2, lower.tail=FALSE) is 2e-20 */
-	long double sum = 0, sum2 = 0, lambda = 0.5*theta, pr = exp(-lambda);
+	LDOUBLE sum = 0, sum2 = 0, lambda = 0.5*theta, 
+	    pr = EXP(-lambda); // does this need a feature test?
 	double ans;
 	int i;
 	/* we need to renormalize here: the result could be very close to 1 */
@@ -124,7 +132,7 @@ pnchisq_raw(double x, double f, double theta,
 #endif
 
     if(f2 * DBL_EPSILON > 0.125 && /* very large f and x ~= f: probably needs */
-       fabs(t = x2 - f2) <         /* another algorithm anyway */
+       FABS(t = x2 - f2) <         /* another algorithm anyway */
        sqrt(DBL_EPSILON) * f2) {
 	/* evade cancellation error */
 	/* t = exp((1 - t)*(2 - t/(f2 + 1))) / sqrt(2*M_PI*(f2 + 1));*/
@@ -151,7 +159,7 @@ pnchisq_raw(double x, double f, double theta,
 	ans = term = 0.; t = 0;
     }
     else {
-	t = exp(lt);
+	t = EXP(lt);
 #ifdef DEBUG_pnch
  	REprintf(", t=exp(lt)= %g\n", t);
 #endif
@@ -201,7 +209,7 @@ pnchisq_raw(double x, double f, double theta,
                 REprintf(" n=%d; nomore underflow in u = exp(lu) ==> change\n",
 			 n);
 #endif
-                v = u = exp(lu); /* the first non-0 'u' */
+                v = u = EXP(lu); /* the first non-0 'u' */
                 lamSml = FALSE;
             }
         } else {
@@ -216,7 +224,7 @@ pnchisq_raw(double x, double f, double theta,
                 REprintf("  n=%d; nomore underflow in t = exp(lt) ==> change\n",
 			 n);
 #endif
-                t = exp(lt); /* the first non-0 't' */
+                t = EXP(lt); /* the first non-0 't' */
                 tSml = FALSE;
             }
         } else {
