@@ -1,7 +1,7 @@
 /*
  *  Mathlib : A C Library of Special Functions
  *  Copyright (C) 1998   Ross Ihaka
- *  Copyright (C) 2000   The R Development Core Team
+ *  Copyright (C) 2000   The R Core Team
  *  Copyright (C) 2005-6 The R Foundation
  *
  *  This version is based on a suggestion by Morten Welinder.
@@ -16,7 +16,7 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  A copy of the GNU General Public License is available via WWW at
+ *  You should have received a copy of the GNU General Public License
  *  http://www.gnu.org/copyleft/gpl.html.
  *
  *  DESCRIPTION
@@ -54,9 +54,17 @@ double qcauchy(double p, double location, double scale,
 	    p = -expm1(p);
 	} else
 	    p = exp(p);
-    } else if (p == 1.)
-	return my_INF;
+    } else {
+	if (p > 0.5) {
+	    if (p == 1.)
+		return my_INF;
+	    p = 1 - p;
+	    lower_tail = !lower_tail;
+	}
+    }
 
+    if (p == 0.5) return location; // avoid 1/Inf below
+    if (p == 0.) return location + (lower_tail ? scale : -scale) * ML_NEGINF; // p = 1. is handled above
     return location + (lower_tail ? -scale : scale) / tan(M_PI * p);
     /*	-1/tan(pi * p) = -cot(pi * p) = tan(pi * (p - 1/2))  */
 }
