@@ -29,7 +29,7 @@ stopifnot(
        apply(arr, 1:2, sum) == t(apply(arr, 2:1, sum)),
        aa == apply(aa,2:3,function(x) x),
        all.equal(apply(apply(aa,2:3, sum),2,sum),
-                 10+16*0:4, tol=4*.Machine$double.eps)
+                 10+16*0:4, tolerance = 4*.Machine$double.eps)
 )
 marg <- list(1:2, 2:3, c(2,4), c(1,3), 2:4, 1:3, 1:4)
 for(m in marg) print(apply(arr, print(m), sum))
@@ -1889,7 +1889,7 @@ fit2 <- glm(counts ~ outcome + treatment, family = poisson,
            data = d.AD, weights = c(0, rep(1,8)), y = FALSE)
 for(z in c("response", "working", "deviance", "pearson"))
     stopifnot(all.equal(residuals(fit, type=z), residuals(fit2, type=z),
-                        scale = 1, tol = 1e-10))
+                        scale = 1, tolerance = 1e-10))
 
 ## apply on arrays with zero extents
 ## Robin Hankin, R-help, 2006-02-13
@@ -2608,14 +2608,10 @@ is.unsorted(data.frame(x=3:4, y=1:2))
 ## R < 2.15.1 got these as FALSE, TRUE, FALSE.
 
 
-## Error in constructing the error message
-assertErrorPrint <- function(expr) {
-    stopifnot(inherits(e <- tryCatch(expr, error=function(e)e), "error"))
-    cat("Asserted Error:", e[["message"]],"\n")
-}
 library("methods")# (not needed here)
-assertErrorPrint( getMethod(ls, "bar", fdef=ls) )
-assertErrorPrint( getMethod(show, "bar") )
+assertCondition <- tools::assertCondition
+assertCondition( getMethod(ls, "bar", fdef=ls), "error", verbose=TRUE)
+assertCondition( getMethod(show, "bar"),        "error", verbose=TRUE)
 ## R < 2.15.1 gave
 ##   cannot coerce type 'closure' to vector of type 'character'
 
@@ -2695,4 +2691,18 @@ str(d)
 
 ## PR#15299 : adding a simple vector to a classed object produced a bad result:
 1:2 + table(1:2)
-## Printed the class attribute in R <= 3.0.0
\ No newline at end of file
+## Printed the class attribute in R <= 3.0.0
+
+## PR#15311 : regmatches<- mishandled regexpr results.
+  x <- c('1', 'B', '3')
+  m <- regexpr('\\d', x)
+  regmatches(x, m) <- c('A', 'C')
+  print(x)
+## Gave a warning and a wrong result up to 3.0.1
+
+## Bad warning found by Radford Neal
+  saveopt <- options(warnPartialMatchDollar=TRUE)
+  pl <- pairlist(abc=1, def=2)
+  pl$ab
+  if (!is.null(saveopt[["warnPartialMatchDollar"]])) options(saveopt)
+## 'abc' was just ''
